C   11/08/90 101231642  MEMBER NAME  INIT     (ZWMS.S)      F77
C-----------------------------------------------------------------------
C.
C. R. Hamberg, W.L. van Neerven and T. Matsuura, Nucl. Phys. B359 (1991) 343.
C.
C-----------------------------------------------------------------------
C.
      SUBROUTINE INIT
C.----------------------------------------------------------------------
C.
C.    Initialization of input parameters.
C.
C.    parameters that may be changed by the user :
C.
C.    MZ     : mass of Z boson
C.    MW     : mass of W boson
C.    GAMZ   : width of Z boson
C.    GAMW   : width of W boson
C.    SINW   : Weinberg angle      : SINW = ( sin(theta_W) )**2
C.
C.    SINC   : Cabibbo mixing angle : SINC = ( sin(theta_C) )**2
C.    NF     : NF is the number of flavours used in the final state,
C.             in the loop diagrams and in the running coupling constant
C.             The flavours are numbered in the following way :
C.                     NF      : 1  2  3  4  5  6
C.                     flavour : d  u  s  c  b  t
C.             In the initial state we use 4 flavours, this is fixed
C.             by the subroutines PDQQB, PDQG, PDQQ and PDQQBS.
C.    GF     : Fermi coupling constant
C.    CONVER : conversion factor.
C.             The value of CONVER is now fixed for the conversion :
C.                         Gev**(-2) ----> nbarn
C.    ALPHA  : fine structure constant
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 MZ,MW
      REAL*8 NF
C.
      COMMON/CASI/CF,CA,NF
      COMMON/CONST/GF,CONVER,ALPHA
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/MASS/MZ,MW
      COMMON/WIDTH/GAMZ,GAMW
      COMMON/ANGLE/SINW,COSW,SINC,COSC
      COMMON/VACOUP/VUCT,AUCT,VDSB,ADSB
      COMMON/PDZ/DELU,DELD
C.
C.    Vector boson masses
C.
CJS(DATA FILE)      MZ=91.196D0
CJS(DATA FILE)      MW=80.24D0
C.
C.    Width of the vector bosons
C.
CJS(DATA FILE)      GAMZ=2.492D0
CJS(DATA FILE)      GAMW=1.927D0
C.
C.    Weinberg angle
C.    SINW = ( sin(theta_W) )**2 = 1.0-(MW/MZ)**2
C.
CJS      SINW=1.0D0 - (MW/MZ)**2
CJS      COSW=1.0D0 - SINW
C.
C.    Cabibbo mixing angle
C.    SINC = ( sin(theta_C) )**2 = 0.05
C.
CJS(DATA FILE)      SINC=0.05D0
CJS      COSC=1.0D0-SINC
C.
C.    V-A couplings for the Z boson
C.
      VUCT= 1.0D0 - 8.0D0/3.0D0*SINW
      AUCT=-1.0D0
      VDSB=-1.0D0 + 4.0D0/3.0D0*SINW
      ADSB= 1.0D0
C.
C.    Factors used for Z-production : DELU & DELD
C.
      DELU=VUCT**2 + AUCT**2
      DELD=VDSB**2 + ADSB**2
C.
C.    Colour factors
C.    CF=(N**2-1)/(2*N), CA=N, N=#colours
C.    We use N=3
C.
CJS      CF=4.0D0/3.0D0
CJS      CA=3.0D0
C.
C.    NF=#flavours
C.    NF is the number of flavours in the final state, in the loop
C.    diagrams and in the formula for the running coupling constant.
C.
CJS(DATA FILE)      NF=4.D0
C.
C.    Fermi coupling constant
C.    GF=1.16639*10**-5 GeV**-2
C.
CJS(DATA FILE)      GF=1.16639D-5
C.
C.    Conversion factor for GeV**-2 ---> nbarn
C.    1 GEV**-2 = 389379.29 NBARN
C.
CJS      CONVER=389379.29D0
C.
C.    fine structure constant
C.
CJS(DATA FILE)      ALPHA=0.7297353D-2 
C.
      PI=4.0D0*DATAN(1.0D0)
C.
C.    Riemann zeta functions
C.
      ZETA2=PI**2/6.0D0
      ZETA3=1.2020569031D0
      END

      SUBROUTINE HEAD
C.----------------------------------------------------------------------
C.
C.    HEAD prints information about the input parameters to UNIT=6
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 MZ,MW
      REAL*8 LAMBDA
      REAL*8 NF
      CHARACTER PDF*9,COEF*10,COUCOM*10,FERFLG*6,BSTR*11,TSTR*7
C.
      COMMON/MASS/MZ,MW
      COMMON/WIDTH/GAMZ,GAMW
      COMMON/ANGLE/SINW,COSW,SINC,COSC
      COMMON/ENERGY/S,TAU
      COMMON/ZORW/ZPROD
      COMMON/WMORWP/WMINUS,WBOTH
      LOGICAL ZPROD,WMINUS,WBOTH
      COMMON/ORDE/NORDER
      COMMON/KIESPD/MODE
      COMMON/SCALES/FACSCA,RENSCA
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/KOPPAR/LAMBDA,KOPAPP
      COMMON/CASI/CF,CA,NF
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/NORM/FERMI
      LOGICAL FERMI
      INTEGER BEAM,TARGET
      COMMON/COLLISION/BEAM,TARGET

C.
      IF (ZPROD) THEN
!        WRITE(6,*) 'Z production'
      ELSE
        IF (WBOTH) THEN
!          WRITE(6,*) 'W(minus) + W(plus) production'
        ELSE
          IF (WMINUS) THEN
!            WRITE(6,*) 'W(minus) production'
          ELSE
!            WRITE(6,*) 'W(plus) production'
          ENDIF
        ENDIF
      ENDIF

!      IF (BEAM.EQ.1) BSTR='proton     '
!      IF (BEAM.EQ.2) BSTR='anti-proton'
!      IF (BEAM.EQ.3) BSTR='neutron'

!      IF (TARGET.EQ.1) TSTR='proton '
!      IF (TARGET.EQ.2) TSTR='neutron'

!      WRITE(6,*) bstr//' '//tstr//' collider'

      WRITE(6,*) 'MSbar mass factorization scheme'

      IF (MODE.EQ.1) PDF='CTEQ3M   '
      IF (MODE.EQ.2) PDF='CTEQ3L   '
      IF (MODE.EQ.4) PDF='MRSR1FIT '
      IF (MODE.EQ.5) PDF='MRSR2FIT '
      IF (MODE.EQ.10) PDF='GRV92LO '
      IF (MODE.EQ.11) PDF='GRV92HO '
      IF (MODE.EQ.12) PDF='GRV94LO '
      IF (MODE.EQ.13) PDF='GRV94HO '
      IF (MODE.EQ.15) PDF='GRV98LO '
      IF (MODE.EQ.16) PDF='GRV98MS '

      IF(NORDER.EQ.0) COEF='LO'
      IF(NORDER.EQ.1) COEF='NLO'
      IF(NORDER.EQ.2) COEF='NNLO'

      IF (KOPAPP.EQ.1) THEN
        COUCOM=' LO'
      ELSE
        COUCOM=' NLO'
      ENDIF

      IF (FERMI) THEN
         FERFLG='TRUE '
      ELSE
         FERFLG='FALSE'
      ENDIF
      WRITE(6,10) !SQRT(S)/1000.0D0,
     ,   MZ,GAMZ,MW,GAMW,!DSQRT(S*TAU),TAU,
!     .            FACSCA,RENSCA,
     ,   SINW,SINC,INT(NF),!LAMBDA,
!     .            PDF,
!     ,            COEF,COUCOM,4.0D0*PI*ALSQ
     ,             FERFLG
10    FORMAT(' Input parameters  :',/
!     .      ,' c.m. energy       : ',F6.2,' TeV',/
     .      ,' Z mass            : ',F6.2,' GeV',/
     .      ,' width of Z boson  : ',F6.2,' GeV',/
     .      ,' W mass            : ',F6.2,' GeV',/
     .      ,' width of W boson  : ',F6.2,' GeV',/
!     .      ,' lepton pair mass  : ',F6.2,' GeV',/
!     .      ,' tau               : ',E10.3,/
!     .      ,' Factor. scale     : ',F6.2,' GeV',/
!     .      ,' Renorm. scale     : ',F6.2,' GeV',/
     .      ,' sin(thW)**2       : ',F6.3,/
     .      ,' sin(thC)**2       : ',F6.3,/
     .      ,' #flavours (n_f)   : ',I2,/
!     .      ,' QCD scale         : ',F6.3,' GeV',/
!     .      ,' pdf               : ',A9,/
!     .      ,' appr. for coef.   : ',A10,/
!     .      ,' appr. for alpha_s : ',A10,/
!     .      ,' alpha_s           : ',E10.4,/
     .      ,' flag FERMI        : ',A6,/)
      RETURN
      END

      DOUBLE PRECISION FUNCTION COUPL(SCALE)
C.----------------------------------------------------------------------
C.
C.    The running coupling constant.
C.
C.    COUPL(Q) = alpha_s(Q)
C.
C.    The scale Q should be given in GeV.
C.
C.    For KOPAPP=1  : leading order formula is used
C.              =2  : next-to-leading order formula is used
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)

      include 'CONSTCOM.'
      INCLUDE 'PDFCOM.'

!  The variation of the scale is taken into account in the grid 
!  therefore turn it back at this place 
 
      q2=scale**2
      coupl=xqg(0,0.1d0,q2/rscale,kschemepdf)

      RETURN
      END

      DOUBLE PRECISION FUNCTION CQB1(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 1st order, q qb, singular terms
C.
C.    To include the factorization scale dependent terms the flag
C.    NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      D0=1.0D0/(1.0D0-Z)
      D1=D0*DLOG(1.0D0-Z)
      HELP=16.0D0*D1
      IF (.NOT.NATFRS) HELP=HELP + 8.0D0 * FACLM * D0
      CQB1=CF * HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQB1R(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 1st order, q qb, regular terms.
C.
C.    To include the factorization scale dependent terms the flag
C.    NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      HELP=-8.0D0*( 1.0D0 + Z )*DLOG(1.0D0-Z)
     .     -4.0D0*( 1.0D0 + Z**2 )*DLOG(Z)/(1.0D0-Z)
      IF (.NOT.NATFRS) HELP=HELP - 4.0D0*( 1.0D0 + Z )*FACLM
      CQB1R=CF * HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQG1(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 1st order, q g
C.
C.    To include the factorization scale dependent terms the flag
C.    NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOG1MZ
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      ZMIN=1.0D0-Z
      LOG1MZ=DLOG(ZMIN)
      HELP=( Z**2 + ZMIN**2 )*( 2.0D0*LOG1MZ - DLOG(Z) ) +
     .     0.5D0 + 3.0D0*Z - 3.5D0*Z**2
      IF (.NOT.NATFRS) HELP=HELP + ( Z**2 + ZMIN**2 )*FACLM
      CQG1=HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQBCF(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q qb, CF**2, singular terms
C.
C.    To include the factorization scale dependent terms the flag
C.    NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOG1MZ
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      LOG1MZ=DLOG(1.0D0-Z)
      D0=1.0D0/(1.0D0-Z)
      D1=D0*LOG1MZ
      D2=D1*LOG1MZ
      D3=D2*LOG1MZ
      HELP=128.0D0*D3 - ( 128.0D0*ZETA2 + 256.0D0 )*D1 +
     .     256.0D0*ZETA3*D0
      IF (.NOT.NATFRS) THEN
        HELP=HELP + 192.0D0*FACLM*D2 +
     .       ( 64.0D0*FACLM + 96.0D0 )*FACLM*D1 +
     .       ( 48.0D0*FACLM - 128.0D0 - 64.0D0*ZETA2 )*FACLM*D0
      ENDIF
      CQBCF=CF**2 * HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQBCFR(Z)
C.---------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q qb, CF*CF, regular terms.
C.
C.    To include the factorization scale dependent terms the flag
C.    NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.---------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ,LOG1MZ
      COMPLEX*16 WGPLG
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      LOGZ=DLOG(Z)
      ZMIN=1.0D0-Z
      LOG1MZ=DLOG(ZMIN)
      DI1MZ=WGPLG(1,1,ZMIN)
      TRI1MZ=WGPLG(2,1,ZMIN)
      S1MZ=WGPLG(1,2,ZMIN)
      HELP=( 1.0D0 + Z ) *
     .          ( 48.0D0*S1MZ - 32.0D0*TRI1MZ - 128.0D0*ZETA3 +
     .            24.0D0*LOG1MZ*DI1MZ + 24.0D0*LOGZ*DI1MZ +
     .            64.0D0*LOG1MZ*ZETA2 -96.0D0*LOGZ*ZETA2
     .           -64.0D0*LOG1MZ**3 + 156.0D0*LOG1MZ**2*LOGZ
     .           -96.0D0*LOG1MZ*LOGZ**2 + 50.0D0/3.0D0*LOGZ**3 ) +
     .     (-64.0D0*S1MZ - 16.0D0*TRI1MZ +
     .       48.0D0*LOG1MZ*DI1MZ - 48.0D0*LOGZ*DI1MZ +
     .       128.0D0*LOGZ*ZETA2
     .      -248.0D0*LOG1MZ**2*LOGZ + 144.0D0*LOG1MZ*LOGZ**2
     .      -24.0D0*LOGZ**3 + 112.0D0*LOGZ )/ZMIN +
     .     DI1MZ*( -24.0D0 - 32.0D0*Z ) + ZETA2*( 64.0D0 - 64.0D0*Z ) +
     .     LOG1MZ**2*( - 64.0D0 + 64.0D0*Z ) +
     .     LOG1MZ*LOGZ*( 64.0D0 - 112.0D0*Z ) +
     .     LOGZ**2*( - 8.0D0 + 24.0D0*Z ) +
     .     LOG1MZ*( 256.0D0 + 12.0D0*Z ) +
     .     LOGZ*( - 104.0D0 + 48.0D0*Z ) - 72.0D0 + 48.0D0*Z
      IF (.NOT.NATFRS) THEN
        HELP = HELP +
     .    FACLM**2*(  LOGZ * ( 24.D0 + 24.D0*Z - 32.D0/ZMIN )
     .             + LOG1MZ * (  - 32.D0 - 32.D0*Z )
     .             - 40.D0 - 8.D0*Z                          ) +
     .    FACLM   *( LOGZ**2 * (  - 36.D0 - 36.D0*Z + 48.D0/ZMIN )
     .             + LOGZ*LOG1MZ * ( 144.D0 + 144.D0*Z - 224.D0/ZMIN )
     .              + LOGZ * ( 56.D0 - 24.D0*Z - 48.D0/ZMIN )
     .              + LOG1MZ**2 * (  - 96.D0 - 96.D0*Z )
     .              + LOG1MZ * (  - 112.D0 + 16.D0*Z )
     .              + DI1MZ * ( 16.D0 + 16.D0*Z + 32.D0/ZMIN )
     .              + ZETA2 * ( 32.D0 + 32.D0*Z )
     .              + 120.D0 + 16.D0*Z                                )
      ENDIF
      CQBCFR=CF*CF*HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQBCA(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q qb, CA*CF, singular terms
C.
C.    To include the factorization and renormalization scale dependent
C.    terms, the flag NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOG1MZ
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      LOG1MZ=DLOG(1.0D0-Z)
      D0=1.0D0/(1.0D0-Z)
      D1=D0*LOG1MZ
      D2=D1*LOG1MZ
      HELP=-176.0D0/3.0D0*D2 + ( 1072.0D0/9.0D0 - 32.0D0*ZETA2 )*D1 +
     .     ( 56.0D0*ZETA3 + 176.0D0/3.0D0*ZETA2 -1616.0D0/27.0D0 )*D0
      IF (.NOT.NATFRS) THEN
        HELP=HELP - 176.0D0/3.0D0*RENLR*D1 +
     .       ( 44.0D0/3.0D0*FACLM**2 - 88.0D0/3.0D0*FACLM*RENLR +
     .         ( 536.0D0/9.0D0 - 16.0D0*ZETA2 )*FACLM )*D0
      ENDIF
      CQBCA=CA*CF * HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQBCAR(Z)
C.---------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q qb, CA*CF, regular terms.
C.
C.    To include the factorization and renormalization scale dependent
C.    terms, the flag NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.---------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 WGPLG
      REAL*8 LOGZ,LOG1MZ
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      LOGZ=DLOG(Z)
      ZMIN=1.0D0-Z
      LOG1MZ=DLOG(ZMIN)
      DI1MZ=WGPLG(1,1,ZMIN)
      TRI1MZ=WGPLG(2,1,ZMIN)
      S1MZ=WGPLG(1,2,ZMIN)
      HELP=( 1.0D0 + Z ) *
     .          ( 20.0D0*S1MZ - 28.0D0*ZETA3 +
     .            16.0D0*LOG1MZ*DI1MZ - 8.0D0*LOGZ*DI1MZ +
     .            16.0D0*LOG1MZ*ZETA2 - 8.0D0*LOGZ*ZETA2 +
     .            88.0D0/3.0D0*LOG1MZ**2 ) +
     .     (-8.0D0*S1MZ - 24.0D0*TRI1MZ
     .      -16.0D0*LOG1MZ*DI1MZ + 16.0D0*LOGZ*DI1MZ +
     .       16.0D0*LOGZ*ZETA2 + 8.0D0/3.0D0*DI1MZ +
     .       280.0D0/3.0D0*LOG1MZ*LOGZ - 29.0D0*LOGZ**2
     .      -208.0D0/3.0D0*LOGZ )/ZMIN
     .      -DI1MZ*( 32.0D0/3.0D0 + 56.0D0/3.0D0*Z )
     .      -ZETA2*( 76.0D0/3.0D0 + 100.0D0/3.0D0*Z )
     .      -LOG1MZ*LOGZ*( 176.0D0/3.0D0 + 176.0D0/3.0D0*Z ) +
     .       LOGZ**2*( 55.0D0/3.0D0 + 55.0D0/3.0D0*Z )
     .      -LOG1MZ*( 152.0D0/9.0D0 + 956.0D0/9.0D0*Z ) +
     .       LOGZ*( 52.0D0/3.0D0 + 218.0D0/3.0D0*Z )
     .      -446.0D0/27.0D0 + 2278.0D0/27.0D0*Z
         IF (.NOT.NATFRS) THEN
        HELP = HELP +
     .     FACLM**2*(  - 22.D0/3.D0 - 22.D0/3.D0*Z ) +
     .     FACLM*RENLR * ( 44.D0/3.D0 + 44.D0/3.D0*Z ) +
     .     FACLM   *( LOGZ * (  - 44.D0/3.D0 - 44.D0/3.D0*Z +
     .                            52.D0/3.D0/ZMIN )
     .              + DI1MZ * ( 8.D0 + 8.D0*Z - 16.D0/ZMIN )
     .              + ZETA2 * ( 8.D0 + 8.D0*Z )
     .              - 76/9.D0 - 496.D0/9.D0*Z                   ) +
     .     RENLR   *( LOGZ * (  - 44.D0/3.D0 - 44.D0/3.D0*Z +
     .                            88.D0/3.D0/ZMIN )
     .              + LOG1MZ * ( 88.D0/3.D0 + 88.D0/3.D0*Z )    )
      ENDIF
      CQBCAR=CA*CF*HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQBNF(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q qb, nf*CF, singular terms
C.
C.    To include the factorization and renormalization scale dependent
C.    terms, the flag NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOG1MZ
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      LOG1MZ=DLOG(1.0D0-Z)
      D0=1.0D0/(1.0D0-Z)
      D1=D0*LOG1MZ
      D2=D1*LOG1MZ
      HELP=32.0D0/3.0D0*D2 - 160.0D0/9.0D0*D1 +
     .     ( 224.0D0/27.0D0 - 32.0D0/3.0D0*ZETA2 )*D0
      IF (.NOT.NATFRS) THEN
        HELP=HELP + 32.0D0/3.0D0*RENLR*D1 +
     .       ( -8.0D0/3.0D0*FACLM**2 + 16.0D0/3.0D0*FACLM*RENLR
     .         -80.0D0/9.0D0*FACLM )*D0
      ENDIF
      CQBNF=NF*CF * HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQBNFR(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q qb, nf*CF, regular terms.
C.
C.    To include the factorization and renormalization scale dependent
C.    terms, the flag NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ,LOG1MZ
      COMPLEX*16 WGPLG
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      LOGZ=DLOG(Z)
      ZMIN=1.0D0-Z
      LOG1MZ=DLOG(ZMIN)
      DI1MZ=WGPLG(1,1,ZMIN)
      HELP=( 1.0D0 + Z ) * ( 16.0D0/3.0D0*ZETA2 + 8.0D0/3.0D0*DI1MZ -
     .             10.0D0/3.0D0*LOGZ**2 + 32.0D0/3.0D0* LOG1MZ*LOGZ -
     .             16.0D0/3.0D0*LOG1MZ**2 ) +
     .     ( -64.0D0/3.0D0*LOG1MZ*LOGZ - 8.0D0/3.0D0*DI1MZ +
     .        8.0D0*LOGZ**2 + 40.0D0/3.0D0*LOGZ ) / ZMIN +
     .     LOG1MZ * ( - 16.0D0/9.0D0 + 176.0D0/9.0D0*Z ) +
     .     LOGZ * ( - 4.0D0/3.0D0 - 44.0D0/3.0D0*Z ) +
     .     188.0D0/27.0D0 - 412.0D0/27.0D0*Z
      IF (.NOT.NATFRS) THEN
        HELP = HELP +
     .     FACLM**2*( 4.D0/3.D0 + 4.D0/3.D0*Z ) +
     .     FACLM*RENLR * (  - 8.D0/3.D0 - 8.D0/3.D0*Z ) +
     .     FACLM   *( LOGZ * ( 8.D0/3.D0 + 8.D0/3.D0*Z
     .                       - 16.D0/3.D0/ZMIN )
     .              - 8.D0/9.D0 + 88.D0/9.D0*Z                  ) +
     .     RENLR   *( LOGZ * ( 8.D0/3.D0 + 8.D0/3.D0*Z
     .                       - 16.D0/3.D0/ZMIN )
     .              + LOG1MZ * (  - 16.D0/3.D0 - 16.D0/3.D0*Z ) )
      ENDIF
      CQBNFR=NF*CF*HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQBBB(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q qb-->q qb V, B**2 diagram
C.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ,LOG1PZ
      COMPLEX*16 WGPLG
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
C.
      LOGZ=DLOG(Z)
      LOG1PZ=DLOG(1.0D0+Z)
      DIMZ=WGPLG(1,1,-Z)
      CQBBB =CF * (
     .   LOGZ**2 * ( 4.D0/3.D0 + 4.D0/3.D0*Z**2 + 8.D0/3.D0*Z )
     . + LOGZ*LOG1PZ * (  - 16.D0/3.D0 - 16.D0/3.D0*Z**2
     .                    - 32.D0/3.D0*Z )
     . + LOGZ * ( 4.D0 + 4.D0*Z**2 + 16.D0/3.D0*Z )
     . + DIMZ * (  - 16.D0/3.D0 - 16.D0/3.D0*Z**2 - 32.D0/3.D0*Z )
     . + ZETA2 * (  - 8.D0/3.D0 - 8.D0/3.D0*Z**2 - 16.D0/3.D0*Z )
     . + 20.D0/3.D0 - 20.D0/3.D0*Z**2 )
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQBAB(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q qb-->q qb V,
C.                                interference term : 2*Re(A*B).
C.
C.    This contribution is related to the triangular anomaly diagram and
C.    disappears if complete doublets are taken into account
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ
C.
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
C.
      LOGZ=DLOG(Z)
      ZMIN=1.0D0-Z
      CQBAB =CF * (
     .   LOGZ * (  - 8.D0 + 8.D0*Z + 16.D0/ZMIN )
     . + 24.D0 - 8.D0*Z  )
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQBACD(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q qb --> q qb V
C.                           interference term: -2*Re(A*C) = -2*Re(A*D)
C.
C.    To include the factorization and renormalization scale dependent
C.    terms, the flag NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ,LOG1MZ
      COMPLEX*16 WGPLG
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      LOGZ=DLOG(Z)
      ZMIN=1.0D0-Z
      LOG1MZ=DLOG(ZMIN)
      DI1MZ=WGPLG(1,1,ZMIN)
      TRI1MZ=WGPLG(2,1,ZMIN)
      S1MZ=WGPLG(1,2,ZMIN)
      DELACD=CF * ( CF - CA/2 ) * (
     . ( LOGZ**3 * 16.D0/3.D0  - LOGZ**2*LOG1MZ * 16.D0
     . + LOGZ**2 * 15.D0       - LOGZ*LOG1MZ * 24.D0
     . - LOGZ*DI1MZ * 24.D0   + LOGZ * 24.D0
     . - LOG1MZ*DI1MZ * 32.D0 - DI1MZ * 12.D0
     . + TRI1MZ * 32.D0       - S1MZ * 72.D0          )/ZMIN
     . + LOGZ**3 * (  - 2.D0 - 2.D0*Z )
     . + LOGZ**2*LOG1MZ * ( 8.D0 + 8.D0*Z )
     . + LOGZ**2 * ( 4.D0 + 4.D0*Z )
     . + LOGZ*LOG1MZ * (  - 16.D0 - 16.D0*Z )
     . + LOGZ*DI1MZ * ( 16.D0 + 16.D0*Z )
     . + LOGZ * ( 32.D0 - 30.D0*Z )
     . + LOG1MZ*DI1MZ * ( 16.D0 + 16.D0*Z )
     . + LOG1MZ * (  - 64.D0 + 56.D0*Z )
     . + DI1MZ * (  - 20.D0 - 20.D0*Z )
     . + TRI1MZ * (  - 24.D0 - 24.D0*Z )
     . + S1MZ * ( 36.D0 + 36.D0*Z )
     . + 94.D0 - 78.D0*Z )
      IF (NATFRS) THEN
        FRTERM=0.0D0
      ELSE
        FRTERM=CF * ( CF - CA/2 ) * FACLM * (
     .           LOGZ**2 * ( 4.D0 + 4.D0*Z - 8.D0/ZMIN )
     .         + LOGZ * (  - 8.D0 - 8.D0*Z - 12.D0/ZMIN )
     .         + DI1MZ * ( 8.D0 + 8.D0*Z - 16.D0/ZMIN )
     .         - 32.D0 + 28.D0*Z                 )
      ENDIF
      CQBACD=DELACD+FRTERM
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQBBCD(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q qb --> q qb V
C.                           interference term: -2*Re(B*C) = -2*Re(B*D)
C.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ,LOG1PZ
      COMPLEX*16 WGPLG
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
C.
      LOGZ=DLOG(Z)
      LOG1PZ=DLOG(1.0D0+Z)
      DIMZ=WGPLG(1,1,-Z)
      DI1MZ=WGPLG(1,1,1.0D0-Z)
      TRIMZ=WGPLG(2,1,-Z)
      SMZ=WGPLG(1,2,-Z)
      S1MZ=WGPLG(1,2,1.0D0-Z)
      CQBBCD =CF * ( CF - CA/2 ) * (
     .   LOGZ**3 * ( 4.D0/3.D0 + 4.D0/3.D0*Z**2 + 16.D0/3.D0*Z )
     . + LOGZ**2*LOG1PZ * ( 20.D0 + 20.D0*Z**2 + 40.D0*Z )
     . + LOGZ**2 * ( 12.D0 - 30.D0*Z**2 - 16.D0*Z )
     . + LOGZ*LOG1PZ**2 * (  - 24.D0 - 24.D0*Z**2 - 48.D0*Z )
     . + LOGZ*LOG1PZ * ( 24.D0 + 24.D0*Z**2 + 48.D0*Z )
     . + LOGZ*DI1MZ * ( 16.D0 + 16.D0*Z**2 + 48.D0*Z )
     . + LOGZ*DIMZ * ( 24.D0 + 24.D0*Z**2 + 48.D0*Z )
     . + LOGZ*ZETA2 * ( 8.D0 + 8.D0*Z**2 + 16.D0*Z )
     . + LOGZ * ( 36.D0 + 44.D0*Z )
     . + LOG1PZ*DIMZ * (  - 48.D0 - 48.D0*Z**2 - 96.D0*Z )
     . + LOG1PZ*ZETA2 * (  - 24.D0 - 24.D0*Z**2 - 48.D0*Z )
     . + DI1MZ * ( 36.D0 - 36.D0*Z**2 )
     . + DIMZ * ( 24.D0 + 24.D0*Z**2 + 48.D0*Z )
     . + ZETA2 * ( 12.D0 + 12.D0*Z**2 + 24.D0*Z )
     . + TRIMZ * (  - 8.D0 - 8.D0*Z**2 - 16.D0*Z )
     . + S1MZ * ( 32.D0 + 32.D0*Z**2 + 96.D0*Z )
     . + SMZ * (  - 48.D0 - 48.D0*Z**2 - 96.D0*Z )
     . + 54.D0 - 26.D0*Z**2 - 28.D0*Z )
      RETURN
      END
      DOUBLE PRECISION FUNCTION CQG2(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q g
C.
C.    To include the factorization scale dependent terms the flag
C.    NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ,LOG1MZ,LOG1PZ
      COMPLEX*16 WGPLG
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      LOGZ=DLOG(Z)
      LOG1MZ=DLOG(1.0D0-Z)
      LOG1PZ=DLOG(1.0D0+Z)
      DIMZ=WGPLG(1,1,-Z)
      DI1MZ=WGPLG(1,1,1.0D0-Z)
      TRIMZ=WGPLG(2,1,-Z)
      TRI1MZ=WGPLG(2,1,1.0D0-Z)
      ZCOM=(1.0D0-Z)/(1.0D0+Z)
      TRIPCO=WGPLG(2,1,ZCOM)
      TRIMCO=WGPLG(2,1,-ZCOM)
      S1MZ=WGPLG(1,2,1.0D0-Z)
      DELCA =
     .   LOGZ**3 * (  - 3.D0 - 20.D0/3.D0*Z )
     . + LOGZ**2*LOG1MZ * ( 6.D0 - 4.D0*Z**2 + 28.D0*Z )
     . + LOGZ**2*LOG1PZ * (  - 6 - 12*Z**2 - 12*Z )
     . + LOGZ**2 * ( 5.D0/2.D0 + 173.D0/3.D0*Z**2 + 2.0D0*Z )
     . + LOGZ*LOG1MZ**2 * (  - 2.D0 + 12.D0*Z**2 - 44.D0*Z )
     . + LOGZ*LOG1MZ*LOG1PZ * ( 8.D0 + 16.D0*Z**2 + 16.D0*Z )
     . + LOGZ*LOG1MZ * (  - 10.D0 - 130.D0*Z**2 + 16.D0*Z )
     . + LOGZ*LOG1PZ * (  - 4.D0 - 16.D0*Z**2 - 20.D0*Z )
     . + LOGZ*DI1MZ * ( 8.D0*Z**2 - 28.D0*Z )
     . + LOGZ*DIMZ * (  - 8.D0 - 16.D0*Z**2 - 16.D0*Z )
     . + LOGZ*ZETA2 * (  - 16.D0*Z**2 + 40.D0*Z )
     . + LOGZ * (  - 118.D0/3.D0 + 457.D0/9.D0*Z**2 + 16.D0/3.D0*Z )
     . + LOG1MZ**3 * (  - 13.D0/3.D0 - 26.D0/3.D0*Z**2 + 26.D0/3.D0*Z )
     . + LOG1MZ**2 * (  - 4.D0 + 154.D0/3.D0*Z**2 - 42.D0*Z
     .                  - 16.D0/3.D0/Z )
     . + LOG1MZ*DI1MZ * (  - 28.D0 - 20.D0*Z**2 - 40.D0*Z )
     . + LOG1MZ*DIMZ * ( 8.D0 + 16.D0*Z**2 + 16.D0*Z )
     . + LOG1MZ*ZETA2 * ( 16.D0 + 32.D0*Z**2 - 16.D0*Z )
      DELCA = DELCA
     . + LOG1MZ * ( 70.D0/3.D0 - 74.D0/9.D0*Z**2
     .             - 25.D0/3.D0*Z - 88.D0/9.D0/Z )
     . + DI1MZ * (  - 22.D0 - 88.D0/3.D0*Z**2 - 64.D0*Z - 32.D0/3.D0/Z )
     . + DIMZ * (  - 4.D0 - 16.D0*Z**2 - 20.D0*Z )
     . + ZETA2 * (  - 10.D0 - 214.D0/3.D0*Z**2 + 56.D0*Z
     .              + 16.D0/3.D0/Z )
     . + TRI1MZ * ( 30.D0 + 24.D0*Z**2 + 68.D0*Z )
     . + TRIMZ * ( 4 + 8*Z**2 + 8*Z )
     . + TRIMCO * ( 8.D0 + 16.D0*Z**2 + 16.D0*Z )
     . + TRIPCO * (  - 8.D0 - 16.D0*Z**2 - 16.D0*Z )
     . + S1MZ * (  - 36.D0 - 16.D0*Z**2 - 64.D0*Z )
     . + ZETA3 * ( 2.D0 + 4.D0*Z**2 + 8.D0*Z )
     . - 539.D0/18.D0 - 1837.D0/54.D0*Z**2 + 613.D0/9.D0*Z
     . - 58.D0/27.D0/Z
      DELCA =-CA * DELCA
      DELCF =
     .   LOGZ**3 * ( 17.D0/6.D0 + 26.D0/3.D0*Z**2 - 17.D0/3.D0*Z )
     . + LOGZ**2*LOG1MZ * (  - 12.D0 - 40.D0*Z**2 + 24.D0*Z )
     . + LOGZ**2 * ( 11.D0/4.D0 + Z**2 - 15.D0*Z )
     . + LOGZ*LOG1MZ**2 * ( 21.D0 + 66.D0*Z**2 - 42.D0*Z )
     . + LOGZ*LOG1MZ * (  - 14.D0 - 96.D0*Z**2 + 96.D0*Z )
     . + LOGZ*LOG1PZ * (  - 8.D0 + 24.D0*Z**2 + 16.D0*Z )
     . + LOGZ*DI1MZ * (  - 2.D0 + 4.D0*Z )
     . + LOGZ*DIMZ * ( 8.D0 + 16.D0*Z**2 - 16.D0*Z )
     . + LOGZ*ZETA2 * (  - 12.D0 - 48.D0*Z**2 + 24.D0*Z )
     . + LOGZ * ( 31.D0/2.D0 + 87.D0*Z**2 - 201.D0/2.D0*Z )
     . + LOG1MZ**3 * (  - 35.D0/3.D0 - 70.D0/3.D0*Z**2 + 70.D0/3.D0*Z )
     . + LOG1MZ**2 * ( 23.D0 + 63.D0*Z**2 - 80.D0*Z )
     . + LOG1MZ*DI1MZ * ( 6.D0 + 52.D0*Z**2 - 12.D0*Z )
     . + LOG1MZ*ZETA2 * ( 8.D0 + 16.D0*Z**2 - 16.D0*Z )
      DELCF = DELCF
     . + LOG1MZ * (  - 26.D0 - 88.D0*Z**2 + 135.D0*Z )
     . + DI1MZ * (  9.D0 - 40.D0*Z**2 + 24.D0*Z )
     . + DIMZ * (  - 8.D0 + 24.D0*Z**2 + 16.D0*Z )
     . + ZETA2 * (  - 10.D0 + 24.D0*Z**2 - 4.D0*Z )
     . + TRI1MZ * ( 2.D0 - 36.D0*Z**2 - 4.D0*Z )
     . + TRIMZ * (  - 16.D0 - 32.D0*Z**2 + 32.D0*Z )
     . + S1MZ * ( 22.D0 + 68.D0*Z**2 - 44.D0*Z )
     . + ZETA3 * (  - 50.D0 - 100.D0*Z**2 + 100.D0*Z )
     . + 157.D0/4.D0 + 305.D0/4.D0*Z**2 - 221.D0/2.D0*Z
      DELCF =-CF * DELCF
      IF (NATFRS) THEN
        FRTERM=0.0D0
      ELSE
        HELPCA=
     .    FACLM**2*( LOGZ * ( 2.D0 + 8.D0*Z )
     .             + LOG1MZ * ( 2.D0 + 4.D0*Z**2 - 4.D0*Z )
     .             + ( 14.D0 - 9.D0*Z**2 + 2.D0*Z + 4.D0/Z ) / 3.D0 ) +
     .    FACLM*RENLR*( -11.D0/3.D0 - 22.D0/3.D0*Z**2 + 22.D0/3.D0*Z ) +
     .    FACLM   *( LOGZ**2 * (  - 4.D0 - 12.D0*Z )
     .             + LOGZ*LOG1MZ * ( 4.D0 - 8.D0*Z**2 + 40.D0*Z )
     .             + LOGZ*LOG1PZ * (  - 4.D0 - 8.D0*Z**2 - 8.D0*Z )
     .             + LOGZ * ( 7.D0 + 146.D0*Z**2 + 10.D0*Z ) / 3.D0
     .             + LOG1MZ**2 * ( 6.D0 + 12.D0*Z**2 - 12.D0*Z )
     .             + LOG1MZ * ( 40.D0/3.D0 - 98.D0/3.D0*Z**2 +
     .                          64.D0/3.D0*Z + 16.D0/3.D0/Z )
     .             + DI1MZ * ( 12.D0 + 8.D0*Z**2 + 24.D0*Z )
     .             + DIMZ * (  - 4.D0 - 8.D0*Z**2 - 8.D0*Z )
     .             + ZETA2 * (  - 8.D0 - 16.D0*Z**2 + 8.D0*Z )
     .             - 47.D0/6.D0 - 85.D0/18.D0*Z**2 + 29.D0/3.D0*Z
     .             + 44.D0/9.D0/Z  ) +
     .    RENLR   *( LOGZ * ( 11.D0 + 22.D0*Z**2 - 22.D0*Z ) / 3.D0
     .             + LOG1MZ * ( -22.D0 - 44.D0*Z**2 + 44.D0*Z ) / 3.D0
     .             - 11.D0/6.D0 + 77.D0/6.D0*Z**2 - 11.D0*Z        )
        HELPCF=
     .    FACLM**2*( LOGZ * (  - 3.D0 - 12.D0*Z**2 + 6.D0*Z )
     .             + LOG1MZ * ( 6.D0 + 12.D0*Z**2 - 12.D0*Z )
     .             - 3.D0/2.D0 + 6.D0*Z                            ) +
     .    FACLM   *( LOGZ**2 * ( 4.D0 + 16.D0*Z**2 - 8.D0*Z )
     .             + LOGZ*LOG1MZ * (  - 20.D0 - 64.D0*Z**2 + 40.D0*Z )
     .             + LOGZ * ( 5.D0 + 46.D0*Z**2 - 40.D0*Z )
     .             + LOG1MZ**2 * ( 18.D0 + 36.D0*Z**2 - 36.D0*Z )
     .             + LOG1MZ * (  - 16.D0 - 46.D0*Z**2 + 68.D0*Z )
     .             + DI1MZ * (  - 24.D0*Z**2 )
     .             + ZETA2 * (  - 4.D0 - 8.D0*Z**2 + 8.D0*Z )
     .             + 12.D0 + 11.D0*Z**2 - 34.D0*Z                  )
        HELPNF=
     .    FACLM**2*( - 2.D0/3.D0 - 4.D0/3.D0*Z**2 + 4.D0/3.D0*Z ) +
     .    FACLM*RENLR*( 2.D0/3.D0 + 4.D0/3.D0*Z**2 - 4.D0/3.D0*Z ) +
     .    FACLM   *( LOGZ * ( 2.D0 + 4.D0*Z**2 - 4.D0*Z ) / 3.D0
     .             + LOG1MZ * (  - 4.D0 - 8.D0*Z**2 + 8.D0*Z ) / 3.D0
     .             - 1.D0/3.D0 + 7.D0/3.D0*Z**2 - 2.D0*Z           ) +
     .    RENLR   *( LOGZ * (  - 2.D0 - 4.D0*Z**2 + 4.D0*Z ) / 3.D0
     .             + LOG1MZ * ( 4.D0 + 8.D0*Z**2 - 8.D0*Z ) / 3.D0
     .             + 1.D0/3.D0 - 7.D0/3.D0*Z**2 + 2.D0*Z            )
        FRTERM=CA*HELPCA + CF*HELPCF + NF*HELPNF
      ENDIF
      CQG2=DELCF + DELCA + FRTERM
      RETURN
      END

      DOUBLE PRECISION FUNCTION CGG(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, g g
C.
C.    To include the factorization scale dependent terms the flag
C.    NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ,LOG1MZ,LOG1PZ
      COMPLEX*16 WGPLG
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      LOGZ=DLOG(Z)
      LOG1MZ=DLOG(1.0D0-Z)
      LOG1PZ=DLOG(1.0D0+Z)
      DIMZ=WGPLG(1,1,-Z)
      DI1MZ=WGPLG(1,1,1.0D0-Z)
      TRIMZ=WGPLG(2,1,-Z)
      TRI1MZ=WGPLG(2,1,1.0D0-Z)
      SMZ=WGPLG(1,2,-Z)
      S1MZ=WGPLG(1,2,1.0D0-Z)
      DELCF1=TRI1MZ * ( 16.0D0 + 64.0D0*Z + 64.0D0*Z**2 ) +
     .       TRIMZ * ( - 8.0D0 - 16.0D0*Z + 8.0D0*Z**2 ) +
     .       S1MZ * ( - 8.0D0 - 80.0D0*Z - 56.0D0*Z**2 ) +
     .       SMZ * ( - 16.0D0 - 32.0D0*Z - 16.0D0*Z**2 ) +
     .       ZETA3 * ( - 4.0D0 - 8.0D0*Z + 8.0D0*Z**2 ) +
     .       LOG1MZ*DI1MZ * ( - 16.0D0 - 64.0D0*Z - 64.0D0*Z**2 ) +
     .       LOGZ*DI1MZ * ( - 4.0D0 - 16.0D0*Z - 16.0D0*Z**2 ) +
     .       LOG1PZ*DIMZ * ( - 16.0D0 - 32.0D0*Z - 16.0D0*Z**2 ) +
     .       LOGZ*DIMZ * ( 16.0D0 + 32.0D0*Z + 8.0D0*Z**2 ) +
     .       LOGZ*ZETA2 * ( 12.0D0 + 40.0D0*Z + 40.0D0*Z**2 ) +
     .       LOG1PZ*ZETA2 * ( - 8.0D0 - 16.0D0*Z - 8.0D0*Z**2 ) +
     .       LOG1MZ**2*LOGZ * ( - 8.0D0 - 32.0D0*Z - 32.0D0*Z**2 ) +
     .       LOG1MZ*LOGZ**2 * ( 4.0D0 + 16.0D0*Z + 16.0D0*Z**2 ) +
     .       LOGZ**3 * ( - 2.0D0 - 16.0D0/3.0D0*Z
     .                   - 16.0D0/3.0D0*Z**2 ) +
     .       LOGZ**2*LOG1PZ * ( 12.0D0 + 24.0D0*Z + 12.0D0*Z**2 ) +
     .       LOGZ*LOG1PZ**2 * ( - 8.0D0 - 16.0D0*Z - 8.0D0*Z**2 )
      DELCF2=DI1MZ * ( - 20.0D0 - 16.0D0*Z + 56.0D0*Z**2 ) +
     .       DIMZ * ( 8.0D0 + 8.0D0*Z ) +
     .       ZETA2 * ( 20.0D0 + 36.0D0*Z - 48.0D0*Z**2 ) +
     .       LOG1MZ**2 * ( - 16.0D0 - 32.0D0*Z + 48.0D0*Z**2 ) +
     .       LOG1MZ*LOGZ * ( 4.0D0 + 32.0D0*Z - 16.0D0*Z**2 ) +
     .       LOGZ**2 * ( - 6.0D0 - 14.0D0*Z - 8.0D0*Z**2 ) +
     .       LOGZ*LOG1PZ * ( 8.0D0 + 8.0D0*Z ) +
     .       LOG1MZ * ( 14.0D0 + 120.0D0*Z - 134.0D0*Z**2 ) +
     .       LOGZ * ( - 23.0D0 - 64.0D0*Z + 105.0D0*Z**2 )
     .       - 32.0D0 - 66.0D0*Z + 98.0D0*Z**2
      DELCA=  CA**2 / ( CA**2 - 1.0D0 ) *
     .      ( S1MZ * ( -8.0D0*Z**2 + 16.0D0*Z - 8.0D0 ) +
     .        SMZ * ( 16.0D0*Z**2 + 32.0D0*Z + 16.0D0 ) +
     .        TRIMZ * ( 24.0D0*Z**2 + 48.0D0*Z + 24.0D0 ) +
     .        ZETA3 * ( 16.0D0*Z**2 + 32.0D0*Z + 16.0D0 ) +
     .        DIMZ * LOGZ * ( -24.0D0*Z**2 - 48.0D0*Z - 24.0D0 ) +
     .        DIMZ * LOG1PZ * ( 16.0D0*Z**2 + 32.0D0*Z + 16.0D0 ) +
     .        DIMZ * ( 16.0D0*Z**2 + 32.0D0*Z +16.0D0 )/3.0D0 +
     .        ZETA2 * LOG1PZ * ( 8.0D0*Z**2 + 16.0D0*Z + 8.0D0 ) +
     .        ZETA2 * ( 8.0D0*Z**2 + 16.0D0*Z +8.0D0 )/3.0D0 +
     .        LOGZ**2 * LOG1PZ * ( -12.0D0*Z**2 - 24.0D0*Z - 12.0D0 ) +
     .        LOGZ**2 * ( 50.0D0*Z**2 + 4.0D0*Z - 4.0D0 )/3.0D0 +
     .        LOGZ * LOG1PZ**2 * ( 8.0D0*Z**2 + 16.0D0*Z + 8.0D0 ) +
     .        LOGZ * LOG1PZ * ( 16.0D0*Z**2 + 32.0D0*Z + 16.0D0)/3.0D0 +
     .        LOGZ * ( -50.0D0*Z**2 - 76.0D0/3.0D0*Z - 4.0D0 ) +
     .        191.0D0/3.0D0*Z**2 - 48.0D0*Z - 47.0D0/3.0D0 )
      IF (NATFRS) THEN
        FRTERM=0.0D0
      ELSE
        FRTERM=( LOGZ * ( - 2.0D0 - 8.0D0*Z - 8.0D0*Z**2 )
     .           - 4.0D0 - 8.0D0*Z + 12.0D0*Z**2          ) * FACLM**2 +
     .         ( DI1MZ * ( - 8.0D0 - 32.0D0*Z - 32.0D0*Z**2 ) +
     .           LOG1MZ*LOGZ * ( - 8.0D0 - 32.0D0*Z - 32.0D0*Z**2 ) +
     .           LOGZ**2 * ( 2.0D0 + 8.0D0*Z + 8.0D0*Z**2 ) +
     .           LOG1MZ * ( - 16.0D0 - 32.0D0*Z + 48.0D0*Z**2 ) +
     .           LOGZ * ( 2.0D0 + 16.0D0*Z - 8.0D0*Z**2 ) +
     .           7.0D0 + 60.0D0*Z - 67.0D0*Z**2                ) * FACLM
      ENDIF
      CGG=DELCF1 + DELCF2 + DELCA + FRTERM
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQQ(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q q, non-interference term
C.
C.    To include the factorization scale dependent terms the flag
C.    NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ,LOG1MZ
      COMPLEX*16 WGPLG
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      LOGZ=DLOG(Z)
      LOG1MZ=DLOG(1.0D0-Z)
      DI1MZ=WGPLG(1,1,1.0D0-Z)
      TRI1MZ=WGPLG(2,1,1.0D0-Z)
      S1MZ=WGPLG(1,2,1.0D0-Z)
      PART1=-8.0D0*TRI1MZ + 12.0D0*S1MZ + 8.0D0*LOG1MZ*DI1MZ +
     .       2.0D0*LOGZ*DI1MZ - 4.0D0*LOGZ*ZETA2 + 1.5D0*LOGZ**3
     .      -4.0D0*LOGZ**2*LOG1MZ + 4.0D0*LOGZ*LOG1MZ**2
      PART1=( 1.0D0 + Z )*PART1
      PART2=DI1MZ * ( 13.0D0 + 8.0D0/3.0D0*Z**2 + 5.0D0*Z +
     .                16.0D0/3.0D0/Z )
     .    + ZETA2 * ( - 2.0D0 + 8.0D0/3.0D0*Z**2 + 2.0D0*Z
     .                - 8.0D0/3.0D0/Z )
     .    + LOG1MZ**2 * ( 2.0D0 - 8.0D0/3.0D0*Z**2 - 2.0D0*Z +
     .                    8.0D0/3.0D0/Z )
     .    + LOGZ*LOG1MZ * ( 6.0D0 + 8.0D0*Z**2 + 12.0D0*Z )
     .    + LOGZ**2 * ( - 5.0D0/4.0D0 - 10.0D0/3.0D0*Z**2
     .                  - 25.0D0/4.0D0*Z )
     .    + LOG1MZ * ( - 26.0D0/3.0D0 - 44.0D0/9.0D0*Z**2 +
     .                   26.0D0/3.0D0*Z + 44.0D0/9.0D0/Z )
     .    + LOGZ * ( 115.0D0/6.0D0 + 10.0D0/9.0D0*Z**2
     .             - 8.0D0/3.0D0*Z )
     .    + 593.0D0/36.0D0 + 703.0D0/108.0D0*Z**2
     .    - 433.0D0/18.0D0*Z + 29.0D0/27.0D0/Z
      IF (NATFRS) THEN
        FRTERM=0.0D0
      ELSE
        FRTERM=FACLM**2 * ( LOGZ * ( 1.0D0 + Z ) +
     .                      0.5D0 - 2.0D0/3.0D0*Z**2 - 0.5D0*Z +
     .                      2.0D0/3.0D0/Z ) +
     .         FACLM * ( DI1MZ * ( 4.0D0 + 4.0D0*Z ) +
     .                   LOGZ**2 * ( - 2.0D0 - 2.0D0*Z ) +
     .                   LOGZ*LOG1MZ * ( 4.0D0 + 4.0D0*Z ) +
     .                   LOGZ * ( 3.0D0 + 4.0D0*Z**2 + 6.0D0*Z ) +
     .                   LOG1MZ * (  2.0D0 - 8.0D0/3.0D0*Z**2
     .                             - 2.0D0*Z + 8.0D0/3.0D0/Z )
     .                  -13.0D0/3.0D0 - 22.0D0/9.0D0*Z**2 +
     .                   13.0D0/3.0D0*Z + 22.0D0/9.0D0/Z )
      ENDIF
      CQQ=( CA**2 - 1.0D0 )/ CA *( PART1 + PART2 + FRTERM )
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQQIV(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q q, interference term,
C.                                no identical particles,
C.                                vector-vector part
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ,LOG1PZ
      COMPLEX*16 WGPLG
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
C.
      LOGZ=DLOG(Z)
      LOG1PZ=DLOG(1.0D0+Z)
      DI1MZ=WGPLG(1,1,1.0D0-Z)
      DIMZ=WGPLG(1,1,-Z)
      TRI1MZ=WGPLG(2,1,1.0D0-Z)
      TRIMZ=WGPLG(2,1,-Z)
      S1MZ=WGPLG(1,2,1.0D0-Z)
      SMZ=WGPLG(1,2,-Z)
      HELP=
     .   LOGZ**3 * ( 4.D0/3.D0*Z )
     . + LOGZ**2*LOG1PZ * (  - 20.D0 - 10.D0*Z - 20.D0/Z )
     . + LOGZ**2 * ( 13.D0*Z )
     . + LOGZ*LOG1PZ**2 * ( 24.D0 + 12.D0*Z + 24.D0/Z )
     . + LOGZ*LOG1PZ * (  - 20.D0 - 20.D0*Z )
     . + LOGZ*DI1MZ * (  - 20.D0 + 2.D0*Z )
     . + LOGZ*DIMZ * (  - 16.D0*Z - 40.D0/Z )
     . + LOGZ*ZETA2 * (  - 20.D0 - 2.D0*Z )
     . + LOGZ * ( 20.D0 + 16.D0*Z )
     . + LOG1PZ*DIMZ * ( 48.D0 + 24.D0*Z + 48.D0/Z )
     . + LOG1PZ*ZETA2 * ( 24.D0 + 12.D0*Z + 24.D0/Z )
      HELP = HELP
     . + DI1MZ * (  - 10.D0 + 8.D0*Z )
     . + DIMZ * (  - 20.D0 - 20.D0*Z )
     . + ZETA2 * (  - 10.D0 - 10.D0*Z )
     . + TRI1MZ * ( 12.D0 - 6.D0*Z - 8.D0/Z )
     . + TRIMZ * (  - 40.D0 + 12.D0*Z + 40.D0/Z )
     . + S1MZ * (  - 16.D0 - 8.D0*Z - 16.D0/Z )
     . + SMZ * ( 48.D0 + 24.D0*Z + 48.D0/Z )
     . + ZETA3 * (  - 36.D0 + 6.D0*Z + 24.D0/Z )
     . + 40.D0 - 40.D0*Z
      CQQIV=CF * HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQQIA(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q q, interference term,
C.                                no identical particles,
C.                                axial-axial part
C.
C.                 !!!!!!    MSbar scheme   !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ,LOG1PZ
      COMPLEX*16 WGPLG
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
C.
      LOGZ=DLOG(Z)
      LOG1PZ=DLOG(1.0D0+Z)
      DI1MZ=WGPLG(1,1,1.0D0-Z)
      DIMZ=WGPLG(1,1,-Z)
      TRI1MZ=WGPLG(2,1,1.0D0-Z)
      TRIMZ=WGPLG(2,1,-Z)
      S1MZ=WGPLG(1,2,1.0D0-Z)
      SMZ=WGPLG(1,2,-Z)
      HELP =
     .   LOGZ**3 * (  - 4.D0/3.D0*Z )
     . + LOGZ**2*LOG1PZ * ( 20.D0 + 10.D0*Z )
     . + LOGZ**2 * (  - Z )
     . + LOGZ*LOG1PZ**2 * (  - 24.D0 - 12.D0*Z )
     . + LOGZ*LOG1PZ * ( 4.D0 + 4.D0*Z )
     . + LOGZ*DI1MZ * ( 4.D0 + 6.D0*Z )
     . + LOGZ*DIMZ * ( 32.D0 )
     . + LOGZ*ZETA2 * ( 4.D0 + 10.D0*Z )  - LOGZ * 4.D0
     . + LOG1PZ*DIMZ * (  - 48.D0 - 24.D0*Z )
     . + LOG1PZ*ZETA2 * (  - 24.D0 - 12.D0*Z )
     . + DI1MZ * ( 2.D0 )
     . + DIMZ * ( 4.D0 + 4.D0*Z )
     . + ZETA2 * ( 2.D0 + 2.D0*Z )
     . + TRI1MZ * ( 4.D0 - 2.D0*Z )
     . + TRIMZ * (  - 24.D0 + 20.D0*Z )
     . + S1MZ * ( 16.D0 + 8.D0*Z )
     . + SMZ * (  - 48.D0 - 24.D0*Z )
     . + ZETA3 * (  - 12.D0 + 18.D0*Z ) - 8.D0 + 8.D0*Z
      CQQIA=CF * HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION CQQCRI(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q q, interference term,
C.                                identical particles in initial state
C.
C.                 !!!!!!   MSbar scheme    !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ
      COMPLEX*16 WGPLG
C.
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
C.
      LOGZ=DLOG(Z)
      DI1MZ=WGPLG(1,1,1.0D0-Z)
      TRI1MZ=WGPLG(2,1,1.0D0-Z)
      S1MZ=WGPLG(1,2,1.0D0-Z)
      CQQCRI=CF * ( CF - CA/2 ) * (
     .   LOGZ**3 * (  - 4.D0/3.D0 - 4.D0/3.D0*Z**2 + 8.D0/3.D0*Z )
     . + LOGZ**2 * (  - 6.D0 - 6.D0*Z**2 + 12.D0*Z )
     . + LOGZ*DI1MZ * (  - 8.D0 - 8.D0*Z**2 + 16.D0*Z )
     . + LOGZ * (  - 14.D0 + 12.D0*Z )
     . + DI1MZ * (  - 12.D0 - 12.D0*Z**2 + 24.D0*Z )
     . + TRI1MZ * ( 8.D0 + 8.D0*Z**2 - 16.D0*Z )
     . + S1MZ * (  - 8.D0 - 8.D0*Z**2 + 16.D0*Z )
     . - 15.D0 - 13.D0*Z**2 + 28.D0*Z )
      RETURN
      END
      DOUBLE PRECISION FUNCTION CQQCRF(Z)
C.----------------------------------------------------------------------
C.
C.    Drell-Yan correction term : 2nd order, q q, interference term
C.                                identical particles in final state
C.
C.    To include the factorization scale dependent terms the flag
C.    NATFRS should be set to .FALSE.
C.
C.                 !!!!!!    MSbar scheme    !!!!!!
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 LOGZ,LOG1MZ,LOG1PZ
      COMPLEX*16 WGPLG
C.
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      ZPLUS=1.0D0+Z
      LOGZ=DLOG(Z)
      LOG1MZ=DLOG(1.0D0-Z)
      LOG1PZ=DLOG(1.0D0+Z)
      DIMZ=WGPLG(1,1,-Z)
      DI1MZ=WGPLG(1,1,1.0D0-Z)
      TRIMZ=WGPLG(2,1,-Z)
      TRI1MZ=WGPLG(2,1,1.0D0-Z)
      ZCOM=(1.0D0-Z)/(1.0D0+Z)
      TRIPCO=WGPLG(2,1,ZCOM)
      TRIMCO=WGPLG(2,1,-ZCOM)
      S1MZ=WGPLG(1,2,1.0D0-Z)
      SMZ=WGPLG(1,2,-Z)
      HELP =
     .   LOGZ**3 * ( 2.D0 - 2.D0*Z - 16.D0/3.D0/ZPLUS )
     . + LOGZ**2*LOG1MZ * (  - 8.D0 + 8.D0*Z + 16.D0/ZPLUS )
     . + LOGZ**2*LOG1PZ * (  - 24.D0 + 24.D0*Z + 56.D0/ZPLUS )
     . + LOGZ**2 * (  - 4.D0 - 12.D0*Z )
     . + LOGZ*LOG1MZ*LOG1PZ * ( 32.D0 - 32.D0*Z - 64.D0/ZPLUS )
     . + LOGZ*LOG1MZ * ( 16.D0 + 16.D0*Z )
     . + LOGZ*LOG1PZ**2 * (  - 16.D0/ZPLUS )
     . + LOGZ*LOG1PZ * ( 8.D0 + 8.D0*Z )
     . + LOGZ*DI1MZ * (  - 24.D0 + 24.D0*Z + 48.D0/ZPLUS )
     . + LOGZ*DIMZ * (  - 32.D0 + 32.D0*Z + 64.D0/ZPLUS )
     . + LOGZ*ZETA2 * (  - 8.D0 + 8.D0*Z + 24.D0/ZPLUS )
     . + LOGZ * (  - 18.D0 + 14.D0*Z )
     . + LOG1MZ*DIMZ * ( 32.D0 - 32.D0*Z - 64.D0/ZPLUS )
     . + LOG1MZ*ZETA2 * ( 16.D0 - 16.D0*Z - 32.D0/ZPLUS )
     . + LOG1MZ * ( 32.D0 - 32.D0*Z )
     . + LOG1PZ*DIMZ * (  - 32.D0/ZPLUS )
     . + LOG1PZ*ZETA2 * (  - 16.D0/ZPLUS )
      HELP = HELP
     . + DI1MZ * ( 24.D0 + 8.D0*Z )
     . + DIMZ * ( 8.D0 + 8.D0*Z )
     . + ZETA2 * ( 4.D0 + 4.D0*Z )
     . + TRI1MZ * ( 32.D0 - 32.D0*Z - 64.D0/ZPLUS )
     . + TRIMZ * ( 16.D0 - 16.D0*Z - 16.D0/ZPLUS )
     . + TRIMCO * ( 32.D0 - 32.D0*Z - 64.D0/ZPLUS )
     . + TRIPCO * (  - 32.D0 + 32.D0*Z + 64.D0/ZPLUS )
     . + S1MZ * (  - 32.D0 + 32.D0*Z + 64.D0/ZPLUS )
     . + SMZ * (  - 32.D0/ZPLUS )
     . + ZETA3 * ( 12.D0 - 12.D0*Z - 8.D0/ZPLUS )
     . - 34.D0 + 34.D0*Z
      DELCAF=CF * ( CF - CA/2 ) * HELP
      IF (NATFRS) THEN
        FRTERM=0.0D0
      ELSE
        FRTERM=CF * ( CF - CA/2 ) * FACLM * (
     .         LOGZ**2 * (  - 4.D0 + 4.D0*Z + 8.D0/ZPLUS )
     .       + LOGZ*LOG1PZ * ( 16.D0 - 16.D0*Z - 32.D0/ZPLUS )
     .       + LOGZ * ( 8.D0 + 8.D0*Z )
     .       + DIMZ * ( 16.D0 - 16.D0*Z - 32.D0/ZPLUS )
     .       + ZETA2 * ( 8.D0 - 8.D0*Z - 16.D0/ZPLUS )
     .       + 16.D0 - 16.D0*Z                 )
      ENDIF
      CQQCRF=DELCAF + FRTERM
      RETURN
      END

      DOUBLE PRECISION FUNCTION PDQQB(X1,X2)
C.----------------------------------------------------------------------
C.
C.    This subroutine contains the quark-antiquark, non-singlet
C.    structure function.
C.    We take into account 4 flavours in the initial state.
C.
C.    The parton distribution functions are combined to form the
C.    structure function belonging to :
C.
C.    ZPROD=.TRUE.  : Z production
C.    ZPROD=.FALSE. : W production
C.
C.    WMINUS=.TRUE.  : W(-) production
C.    WMINUS=.FALSE. : W(+)  production
C.    WBOTH =.TRUE.  : W(-) + W(+) production
C.
C.    BEAM = 1       : X1 corresponds to a proton
C.           2       : X1 corresponds to an anti-proton
C.           3       : X1 corresponds to a neutron
C.
C.    TARGET = 1     : X2 corresponds to a proton
C.           = 2     : X2 corresponds to a neutron
C.
C.    The parton distributions are calculated by the call :
C.
C.    CALL STRUCT(X,SCALE,MODE,UV,DV,USEA,DSEA,STR,CHM,BOT,GLU)
C.
C.    Here : SCALE = scale in GeV
C.           MODE  = specifies the parton distribution function
C.           UV    : up valence quark
C.           DV    : down valence quark
C.           USEA  : up sea quark
C.           DSEA  : down sea quark
C.           STR   : strange quark
C.           CHM   : charm quark
C.           BOT   : bottom quark
C.           GLU   : gluon
C.
C.    For our purposes BOT is a dummy variable.
C.
C.    N.B.: STRUCT returns X * parton distribution function.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/PDZ/DELU,DELD
      COMMON/ANGLE/SINW,COSW,SINC,COSC
      COMMON/ZORW/ZPROD
      LOGICAL ZPROD
      COMMON/WMORWP/WMINUS,WBOTH
      LOGICAL WMINUS,WBOTH
      COMMON/KIESPD/MODE
      COMMON/ENERGY/S,TAU
      COMMON/SCALES/FACSCA,RENSCA
      INTEGER BEAM,TARGET
      COMMON/COLLISION/BEAM,TARGET
C.
      CALL STRUCT(X1,FACSCA,MODE,UV1,DV1,USEA1,DSEA1,
     .                           STR1,CHM1,BOT1,GLU1)
      CALL STRUCT(X2,FACSCA,MODE,UV2,DV2,USEA2,DSEA2,
     .                           STR2,CHM2,BOT2,GLU2)
      IF (BEAM.EQ.1) THEN
        U1=UV1+USEA1
        D1=DV1+DSEA1
        S1=STR1
        C1=CHM1
        UB1=USEA1
        DB1=DSEA1
        SB1=STR1
        CB1=CHM1
      ELSEIF (BEAM.EQ.2) THEN
        U1=USEA1
        D1=DSEA1
        S1=STR1
        C1=CHM1
        UB1=UV1+USEA1
        DB1=DV1+DSEA1
        SB1=STR1
        CB1=CHM1
      ELSEIF (BEAM.EQ.3) THEN
        U1=DV1+DSEA1
        D1=UV1+USEA1
        S1=STR1
        C1=CHM1
        UB1=DSEA1
        DB1=USEA1
        SB1=STR1
        CB1=CHM1
        ENDIF
C.
      IF (TARGET.EQ.1) THEN
        U2=UV2+USEA2
        D2=DV2+DSEA2
        S2=STR2
        C2=CHM2
        UB2=USEA2
        DB2=DSEA2
        SB2=STR2
        CB2=CHM2
      ELSEIF (TARGET.EQ.2) THEN
        U2=DV2+DSEA2
        D2=UV2+USEA2
        S2=STR2
        C2=CHM2
        UB2=DSEA2
        DB2=USEA2
        SB2=STR2
        CB2=CHM2
      ENDIF

      IF (ZPROD) THEN
        HELP1=DELU * ( U1*UB2 + UB1*U2 + C1*CB2 + CB1*C2 )
        HELP2=DELD * ( D1*DB2 + DB1*D2 + S1*SB2 + SB1*S2 )
      ELSE
        IF (WBOTH) THEN
          HELP1=COSC * ( UB1*D2 + D1*UB2 + CB1*S2 + S1*CB2 +
     .                   U1*DB2 + DB1*U2 + C1*SB2 + SB1*C2 )
          HELP2=SINC * ( UB1*S2 + S1*UB2 + CB1*D2 + D1*CB2 +
     .                   U1*SB2 + SB1*U2 + C1*DB2 + DB1*C2 )
        ELSE
          IF (WMINUS) THEN
            HELP1=COSC * ( UB1*D2 + D1*UB2 + CB1*S2 + S1*CB2 )
            HELP2=SINC * ( UB1*S2 + S1*UB2 + CB1*D2 + D1*CB2 )
          ELSE
            HELP1=COSC * ( U1*DB2 + DB1*U2 + C1*SB2 + SB1*C2 )
            HELP2=SINC * ( U1*SB2 + SB1*U2 + C1*DB2 + DB1*C2 )
          ENDIF
        ENDIF
      ENDIF
      PDQQB=( HELP1 + HELP2 ) / X1 / X2
      RETURN
      END

      DOUBLE PRECISION FUNCTION PDQG(X1,X2)
C.----------------------------------------------------------------------
C.
C     This subroutine contains the quark-gluon structure function.
C.    We take into account 4 flavours in the initial state.
C.
C.    The parton distribution functions are combined to form the
C.    structure function belonging to :
C.
C.    ZPROD=.TRUE.  : Z production
C.    ZPROD=.FALSE. : W production
C.
C.    WMINUS=.TRUE.  : W(-) production
C.    WMINUS=.FALSE. : W(+)  production
C.    WBOTH =.TRUE.  : W(-) + W(+) production
C.
C.    BEAM = 1       : X1 corresponds to a proton
C.           2       : X1 corresponds to an anti-proton
C.           3       : X1 corresponds to a neutron
C.
C.    TARGET = 1     : X2 corresponds to a proton
C.           = 2     : X2 corresponds to a neutron
C.
C.    The parton distributions are calculated by the call :
C.
C.    CALL STRUCT(X,SCALE,MODE,UV,DV,USEA,DSEA,STR,CHM,BOT,GLU)
C.
C.    Here : SCALE = scale in GeV
C.           MODE  = specifies the parton distribution function
C.           UV    : up valence quark
C.           DV    : down valence quark
C.           USEA  : up sea quark
C.           DSEA  : down sea quark
C.           STR   : strange quark
C.           CHM   : charm quark
C.           BOT   : bottom quark
C.           GLU   : gluon
C.
C.    For our purposes BOT is a dummy variable.
C.
C.    N.B.: STRUCT returns X * parton distribution function.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/PDZ/DELU,DELD
      COMMON/ZORW/ZPROD
      LOGICAL ZPROD
      COMMON/WMORWP/WMINUS,WBOTH
      LOGICAL WMINUS,WBOTH
      COMMON/KIESPD/MODE
      COMMON/ENERGY/S,TAU
      COMMON/SCALES/FACSCA,RENSCA
      INTEGER BEAM,TARGET
      COMMON/COLLISION/BEAM,TARGET
C.
      CALL STRUCT(X1,FACSCA,MODE,UV1,DV1,USEA1,DSEA1,
     .                           STR1,CHM1,BOT1,GLU1)
      CALL STRUCT(X2,FACSCA,MODE,UV2,DV2,USEA2,DSEA2,
     .                           STR2,CHM2,BOT2,GLU2)
      IF (BEAM.EQ.1) THEN
        U1=UV1+USEA1
        D1=DV1+DSEA1
        S1=STR1
        C1=CHM1
        UB1=USEA1
        DB1=DSEA1
        SB1=STR1
        CB1=CHM1
      ELSEIF (BEAM.EQ.2) THEN
        U1=USEA1
        D1=DSEA1
        S1=STR1
        C1=CHM1
        UB1=UV1+USEA1
        DB1=DV1+DSEA1
        SB1=STR1
        CB1=CHM1
      ELSEIF (BEAM.EQ.3) THEN
        U1=DV1+DSEA1
        D1=UV1+USEA1
        S1=STR1
        C1=CHM1
        UB1=DSEA1
        DB1=USEA1
        SB1=STR1
        CB1=CHM1
      ENDIF
C.
      IF (TARGET.EQ.1) THEN
        U2=UV2+USEA2
        D2=DV2+DSEA2
        S2=STR2
        C2=CHM2
        UB2=USEA2
        DB2=DSEA2
        SB2=STR2
        CB2=CHM2
      ELSEIF (TARGET.EQ.2) THEN
        U2=DV2+DSEA2
        D2=UV2+USEA2
        S2=STR2
        C2=CHM2
        UB2=DSEA2
        DB2=USEA2
        SB2=STR2
        CB2=CHM2
      ENDIF
 
      IF (ZPROD) THEN
        HELP1=DELU * ( ( U1 + UB1 + C1 + CB1 ) * GLU2 +
     .                 GLU1 * ( U2 + UB2 + C2 + CB2 ) )
        HELP2=DELD * ( ( D1 + DB1 + S1 + SB1 ) * GLU2 +
     .                 GLU1 * ( D2 + DB2 + S2 + SB2 ) )
      ELSE
        IF (WBOTH) THEN
          HELP1=( U1 + UB1 + D1 + DB1 + S1 + SB1 + C1 + CB1 ) * GLU2 +
     .          GLU1 * ( U2 + UB2 + D2 + DB2 + S2 + SB2 + C2 + CB2 )
        ELSE
          IF (WMINUS) THEN
            HELP1=( UB1 + D1 + S1 + CB1 ) * GLU2 +
     .            GLU1 * ( UB2 + D2 + S2 + CB2 )
          ELSE
            HELP1=( U1 + DB1 + SB1 + C1 ) * GLU2 +
     .            GLU1 * ( U2 + DB2 + SB2 + C2 )
          ENDIF
        ENDIF
        HELP2=0.0D0
      ENDIF
      PDQG=( HELP1 + HELP2 ) / X1 / X2
      RETURN
      END

      DOUBLE PRECISION FUNCTION PDGG(X1,X2)
C.----------------------------------------------------------------------
C.
C.    This subroutine contains the gluon-gluon structure function.
C.
C.    In this subroutine it is assumed that the flavours are numbered in
C.    the following way :
C.    NF      : 1  2  3  4  5  6
C.    flavour : d  u  s  c  b  t
C.
C.    The parton distribution functions are combined to form the
C.    structure function belonging to :
C.
C.    ZPROD=.TRUE.  : Z production
C.    ZPROD=.FALSE. : W production
C.
C.    WBOTH =.TRUE.  : W(-) + W(+) production
C.
C.    BEAM = 1       : X1 corresponds to a proton
C.           2       : X1 corresponds to an anti-proton
C.           3       : X1 corresponds to a neutron
C.
C.    TARGET = 1     : X2 corresponds to a proton
C.           = 2     : X2 corresponds to a neutron
C.
C.    The parton distributions are calculated by the call :
C.
C.    CALL STRUCT(X,SCALE,MODE,UV,DV,USEA,DSEA,STR,CHM,BOT,GLU)
C.
C.    Here : SCALE = scale in GeV
C.           MODE  = specifies the parton distribution function
C.           UV    : up valence quark
C.           DV    : down valence quark
C.           USEA  : up sea quark
C.           DSEA  : down sea quark
C.           STR   : strange quark
C.           CHM   : charm quark
C.           BOT   : bottom quark
C.           GLU   : gluon
C.
C.    For our purposes BOT is a dummy variable.
C.
C.    N.B.: STRUCT returns X * parton distribution function.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/PDZ/DELU,DELD
      COMMON/ZORW/ZPROD
      COMMON/WMORWP/WMINUS,WBOTH
      LOGICAL ZPROD,WMINUS,WBOTH
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/KIESPD/MODE
      COMMON/ENERGY/S,TAU
      COMMON/SCALES/FACSCA,RENSCA
      INTEGER BEAM,TARGET
      COMMON/COLLISION/BEAM,TARGET
C.
      CALL STRUCT(X1,FACSCA,MODE,UV1,DV1,USEA1,DSEA1,
     .                           STR1,CHM1,BOT1,GLU1)
      CALL STRUCT(X2,FACSCA,MODE,UV2,DV2,USEA2,DSEA2,
     .                           STR2,CHM2,BOT2,GLU2)

      IF (ZPROD) THEN
        HELP=( DFLOAT( INT( (NF+1.0D0)/2.0D0 ) ) * DELD +
     .         DFLOAT( INT( NF/2.0D0 ) ) * DELU          ) * GLU1 * GLU2
      ELSE
        HELP=DFLOAT( INT( NF/2.0D0 ) ) * GLU1 * GLU2
        IF (WBOTH) HELP=2.0D0*HELP
      ENDIF
      PDGG=HELP / X1 / X2
      RETURN
      END

      DOUBLE PRECISION FUNCTION PDQQ(X1,X2,X3)
C.----------------------------------------------------------------------
C.
C.    This subroutine contains the quark-quark and the
C.    antiquark-antiquark structure function.
C.    We take into account 4 flavours in the initial state.
C.
C.    In this case the Drell-Yan correction terms ( CQQ, CQQIV, CQQIA,
C.    CQQCRI and CQQCRF ) are incorporated in the structure function.
C.    X1 and X2 belong to the parton distribution functions,
C.    X3 is used in the Drell-Yan correction terms.
C.
C.    The parton distribution functions are combined to form the
C.    structure function belonging to :
C.
C.    ZPROD=.TRUE.  : Z production
C.    ZPROD=.FALSE. : W production
C.
C.    WMINUS=.TRUE.  : W(-) production
C.    WMINUS=.FALSE. : W(+)  production
C.    WBOTH =.TRUE.  : W(-) + W(+) production
C.
C.    BEAM = 1       : X1 corresponds to a proton
C.           2       : X1 corresponds to an anti-proton
C.           3       : X1 corresponds to a neutron
C.
C.    TARGET = 1     : X2 corresponds to a proton
C.           = 2     : X2 corresponds to a neutron
C.
C.    The parton distributions are calculated by the call :
C.
C.    CALL STRUCT(X,SCALE,MODE,UV,DV,USEA,DSEA,STR,CHM,BOT,GLU)
C.
C.    Here : SCALE = scale in GeV
C.           MODE  = specifies the parton distribution function
C.           UV    : up valence quark
C.           DV    : down valence quark
C.           USEA  : up sea quark
C.           DSEA  : down sea quark
C.           STR   : strange quark
C.           CHM   : charm quark
C.           BOT   : bottom quark
C.           GLU   : gluon
C.
C.    For our purposes BOT is a dummy variable.
C.
C.    N.B.: STRUCT returns X * parton distribution function.
C.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/VACOUP/VUCT,AUCT,VDSB,ADSB
      COMMON/PDZ/DELU,DELD
      COMMON/ANGLE/SINW,COSW,SINC,COSC
      COMMON/ZORW/ZPROD
      LOGICAL ZPROD
      COMMON/WMORWP/WMINUS,WBOTH
      LOGICAL WMINUS,WBOTH
      COMMON/KIESPD/MODE
      COMMON/ENERGY/S,TAU
      COMMON/SCALES/FACSCA,RENSCA
      INTEGER BEAM,TARGET
      COMMON/COLLISION/BEAM,TARGET
C.
      CALL STRUCT(X1,FACSCA,MODE,UV1,DV1,USEA1,DSEA1,
     .                           STR1,CHM1,BOT1,GLU1)
      CALL STRUCT(X2,FACSCA,MODE,UV2,DV2,USEA2,DSEA2,
     .                           STR2,CHM2,BOT2,GLU2)
      IF (BEAM.EQ.1) THEN
        U1=UV1+USEA1
        D1=DV1+DSEA1
        S1=STR1
        C1=CHM1
        UB1=USEA1
        DB1=DSEA1
        SB1=STR1
        CB1=CHM1
      ELSEIF (BEAM.EQ.2) THEN
        U1=USEA1
        D1=DSEA1
        S1=STR1
        C1=CHM1
        UB1=UV1+USEA1
        DB1=DV1+DSEA1
        SB1=STR1
        CB1=CHM1
      ELSEIF (BEAM.EQ.3) THEN
        U1=DV1+DSEA1
        D1=UV1+USEA1
        S1=STR1
        C1=CHM1
        UB1=DSEA1
        DB1=USEA1
        SB1=STR1
        CB1=CHM1
      ENDIF
C.
      IF (TARGET.EQ.1) THEN
        U2=UV2+USEA2
        D2=DV2+DSEA2
        S2=STR2
        C2=CHM2
        UB2=USEA2
        DB2=DSEA2
        SB2=STR2
        CB2=CHM2
      ELSEIF (TARGET.EQ.2) THEN
        U2=DV2+DSEA2
        D2=UV2+USEA2
        S2=STR2
        C2=CHM2
        UB2=DSEA2
        DB2=USEA2
        SB2=STR2
        CB2=CHM2
      ENDIF

      IF (ZPROD) THEN
        HELP1=( U1 + C1 )*( U2 + C2 ) + ( UB1 + CB1 )*( UB2 + CB2 )
        HELP2=( U1 + C1 )*( D2 + S2 ) + ( UB1 + CB1 )*( DB2 + SB2 ) +
     .        ( D1 + S1 )*( U2 + C2 ) + ( DB1 + SB1 )*( UB2 + CB2 )
        HELP3=( D1 + S1 )*( D2 + S2 ) + ( DB1 + SB1 )*( DB2 + SB2 )
        HELP4=2.0D0*DELU*( U1*U2 + C1*C2 + UB1*UB2 + CB1*CB2 )+
     .        2.0D0*DELD*( D1*D2 + S1*S2 + DB1*DB2 + SB1*SB2 )
        HELP=( 2.0D0*DELU*HELP1 + (DELU+DELD)*HELP2 + 2.0D0*DELD*HELP3 )
     .         * CQQ(X3) +
     .       ( 2.0D0*VUCT**2*HELP1 + 2.0D0*VUCT*VDSB*HELP2 +
     .         2.0D0*VDSB**2*HELP3 ) * CQQIV(X3) +
     .       ( 2.0D0*AUCT**2*HELP1 + 2.0D0*AUCT*ADSB*HELP2 +
     .         2.0D0*ADSB**2*HELP3 ) * CQQIA(X3) +
     .       HELP4 * ( CQQCRI(X3) + CQQCRF(X3) )
      ELSE
        IF (WBOTH) THEN
          HELP1=2.0D0*( U1 + D1 + S1 + C1 )*( U2 + D2 + S2 + C2 ) +
     .          2.0D0*( UB1 + DB1 + SB1 + CB1 )*( UB2 + DB2 + SB2 +CB2 )
          HELP2=2.0D0*( UB1*UB2 + CB1*CB2 + D1*D2 + S1*S2 +
     .                  U1*U2 + C1*C2 + DB1*DB2 + SB1*SB2 )
          HELP3=2.0D0*COSC*( UB1*DB2 + CB1*SB2 + D1*U2 + S1*C2 +
     .                       UB2*DB1 + CB2*SB1 + D2*U1 + S2*C1 ) +
     .          2.0D0*SINC*( UB1*SB2 + CB1*DB2 + D1*C2 + S1*U2 +
     .                       UB2*SB1 + CB2*DB1 + D2*C1 + S2*U1 )
        ELSE
          IF (WMINUS) THEN
           HELP1=( UB1 + CB1 )*( DB2 + SB2 ) + ( D1 + S1 )*( U2 + C2 ) +
     .           ( UB2 + CB2 )*( DB1 + SB1 ) + ( D2 + S2 )*( U1 + C1 ) +
     .           2.0D0*( UB1 + CB1 )*( UB2 + CB2 ) +
     .           2.0D0*( D1 + S1 )*( D2 + S2 )
           HELP2=2.0D0*( UB1*UB2 + CB1*CB2 + D1*D2 + S1*S2 )
           HELP3=COSC*( UB1*DB2 + CB1*SB2 + D1*U2 + S1*C2+
     .                  UB2*DB1 + CB2*SB1 + D2*U1 + S2*C1 ) +
     .           SINC*( UB1*SB2 + CB1*DB2 + D1*C2 + S1*U2 +
     .                  UB2*SB1 + CB2*DB1 + D2*C1 + S2*U1 )
          ELSE
           HELP1=( U1 + C1 )*( D2 + S2 ) + ( DB1 + SB1 )*( UB2 + CB2 ) +
     .           ( U2 + C2 )*( D1 + S1 ) + ( DB2 + SB2 )*( UB1 + CB1 ) +
     .           2.0D0*( U1 + C1 )*( U2 + C2) +
     .           2.0D0*( DB1 + SB1 )*( DB2 + SB2 )
           HELP2=2.0D0*( U1*U2 + C1*C2 + DB1*DB2 + SB1*SB2 )
           HELP3=COSC*( U1*D2 + C1*S2 + DB1*UB2 + SB1*CB2 +
     .                  U2*D1 + C2*S1 + DB2*UB1 + SB2*CB1 ) +
     .           SINC*( U1*S2 + C1*D2 + DB1*CB2 + SB1*UB2 +
     .                  U2*S1 + C2*D1 + DB2*CB1 + SB2*UB1 )
         ENDIF
        ENDIF
        HELP=HELP1 * CQQ(X3) + HELP2 * CQQCRI(X3) +
     .       HELP3 * CQQCRF(X3)
      ENDIF
      PDQQ=HELP / X1 / X2
      RETURN
      END

      DOUBLE PRECISION FUNCTION PDQQBS(X1,X2,X3)
C.----------------------------------------------------------------------
C.
C.    This subroutine contains the quark-antiquark, singlet structure
C.    function. Here we take into account the matrixelements related
C.    to the amplitudes B, C and D.
C.    We take into account 4 flavours in the initial state.
C.
C.    In this case the Drell-Yan correction terms ( CQQ, CQQIV, CQQIA
C.    CQBBB, CQBAB and CQBBCD ) are incorporated in the structure
C.    function.
C.    X1 and X2 belong to the parton distribution functions,
C.    X3 is used in the Drell-Yan correction terms.
C.
C.    The parton distribution functions are combined to form the
C.    structure function belonging to :
C.
C.    ZPROD=.TRUE.  : Z production
C.    ZPROD=.FALSE. : W production
C.
C.    WMINUS=.TRUE.  : W(-) production
C.    WMINUS=.FALSE. : W(+)  production
C.    WBOTH =.TRUE.  : W(-) + W(+) production
C.
C.    BEAM = 1       : X1 corresponds to a proton
C.           2       : X1 corresponds to an anti-proton
C.           3       : X1 corresponds to a neutron
C.
C.    TARGET = 1     : X2 corresponds to a proton
C.           = 2     : X2 corresponds to a neutron
C.
C.    The parton distributions are calculated by the call :
C.
C.    CALL STRUCT(X,SCALE,MODE,UV,DV,USEA,DSEA,STR,CHM,BOT,GLU)
C.
C.    Here : SCALE = scale in GeV
C.           MODE  = specifies the parton distribution function
C.           UV    : up valence quark
C.           DV    : down valence quark
C.           USEA  : up sea quark
C.           DSEA  : down sea quark
C.           STR   : strange quark
C.           CHM   : charm quark
C.           BOT   : bottom quark
C.           GLU   : gluon
C.
C.    For our purposes BOT is a dummy variable.
C.
C.    N.B.: STRUCT returns X * parton distribution function.
C.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/VACOUP/VUCT,AUCT,VDSB,ADSB
      COMMON/PDZ/DELU,DELD
      COMMON/ANGLE/SINW,COSW,SINC,COSC
      COMMON/ZORW/ZPROD
      LOGICAL ZPROD
      COMMON/WMORWP/WMINUS,WBOTH
      LOGICAL WMINUS,WBOTH
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/KIESPD/MODE
      COMMON/ENERGY/S,TAU
      COMMON/SCALES/FACSCA,RENSCA
      INTEGER BEAM,TARGET
      COMMON/COLLISION/BEAM,TARGET
C.

      CALL STRUCT(X1,FACSCA,MODE,UV1,DV1,USEA1,DSEA1,
     .                           STR1,CHM1,BOT1,GLU1)
      CALL STRUCT(X2,FACSCA,MODE,UV2,DV2,USEA2,DSEA2,
     .                           STR2,CHM2,BOT2,GLU2)
      IF (BEAM.EQ.1) THEN
        U1=UV1+USEA1
        D1=DV1+DSEA1
        S1=STR1
        C1=CHM1
        UB1=USEA1
        DB1=DSEA1
        SB1=STR1
        CB1=CHM1
      ELSEIF (BEAM.EQ.2) THEN
        U1=USEA1
        D1=DSEA1
        S1=STR1
        C1=CHM1
        UB1=UV1+USEA1
        DB1=DV1+DSEA1
        SB1=STR1
        CB1=CHM1
      ELSEIF (BEAM.EQ.3) THEN
        U1=DV1+DSEA1
        D1=UV1+USEA1
        S1=STR1
        C1=CHM1
        UB1=DSEA1
        DB1=USEA1
        SB1=STR1
        CB1=CHM1
      ENDIF
C.
      IF (TARGET.EQ.1) THEN
        U2=UV2+USEA2
        D2=DV2+DSEA2
        S2=STR2
        C2=CHM2
        UB2=USEA2
        DB2=DSEA2
        SB2=STR2
        CB2=CHM2
      ELSEIF (TARGET.EQ.2) THEN
        U2=DV2+DSEA2
        D2=UV2+USEA2
        S2=STR2
        C2=CHM2
        UB2=DSEA2
        DB2=USEA2
        SB2=STR2
        CB2=CHM2
      ENDIF

      IF (ZPROD) THEN
        HELP1=( U1 + C1 )*( UB2 + CB2 ) + ( UB1 + CB1 )*( U2 + C2 )
        HELP2=( U1 + C1 )*( DB2 + SB2 ) + ( DB1 + SB1 )*( U2 + C2 ) +
     .        ( D1 + S1 )*( UB2 + CB2 ) + ( UB1 + CB1 )*( D2 + S2 )
        HELP3=( D1 + S1 )*( DB2 + SB2 ) + ( DB1 + SB1 )*( D2 + S2 )
        HELP4=U1*UB2 + UB1*U2 + C1*CB2 + CB1*C2
        HELP5=D1*DB2 + DB1*D2 + S1*SB2 + SB1*S2
        HELP=
     .       ( 2.0D0*DELU*HELP1 + (DELU+DELD)*HELP2 + 2.0D0*DELD*HELP3 )
     .         * CQQ(X3) +
     .       ( -2.0D0*VUCT**2*HELP1 - 2.0D0*VUCT*VDSB*HELP2
     .         -2.0D0*VDSB**2*HELP3 ) * CQQIV(X3) +
     .       ( 2.0D0*AUCT**2*HELP1 + 2.0D0*AUCT*ADSB*HELP2 +
     .         2.0D0*ADSB**2*HELP3 ) * CQQIA(X3) +
     .       ( DFLOAT( INT( (NF+1.0D0)/2.0D0 ) )*DELD +
     .         DFLOAT( INT( NF/2.0D0 ) )*DELU      ) *
     .       ( HELP4 + HELP5 ) * CQBBB(X3) +
     .       ( NF - 2.0D0*DFLOAT( INT(NF/2.0D0) ) ) *
     .       ( -HELP4 + HELP5 ) * CQBAB(X3)
     .      +( HELP4*DELU + HELP5*DELD )* 2.0D0 * CQBBCD(X3)
      ELSE
        HELP2=U1*UB2 + UB1*U2 + C1*CB2 + CB1*C2 +
     .        D1*DB2 + DB1*D2 + S1*SB2 + SB1*S2
        IF (WBOTH) THEN
          HELP1=2.0D0*( U1 + D1 + S1 + C1 )*( UB2 + DB2 + SB2 + CB2 ) +
     .          2.0D0*( UB1 + DB1 + SB1 + CB1 )*( U2 + D2 + S2 + C2 )
          HELP2=2.0D0*HELP2
        ELSE
          IF (WMINUS) THEN
           HELP1=( UB1 + CB1 )*( U2 + C2 ) + ( D1 + S1 )*( DB2 + SB2 ) +
     .           ( UB2 + CB2 )*( U1 + C1 ) + ( D2 + S2 )*( DB1 + SB1 ) +
     .           2.0D0*( UB1 + CB1 )*( D2 + S2 ) +
     .           2.0D0*( D1 + S1 )*( UB2 + CB2 )
          ELSE
           HELP1=( U1 + C1 )*( UB2 + CB2 ) + ( DB1 + SB1 )*( D2 + S2 ) +
     .           ( U2 + C2 )*( UB1 + CB1 ) + ( DB2 + SB2 )*( D1 + S1 ) +
     .           2.0D0*( U1 + C1 )*( DB2 + SB2 ) +
     .           2.0D0*( DB1 + SB1 )*( U2 + C2 )
          ENDIF
        ENDIF
        HELP=HELP1*CQQ(X3) +
     .       HELP2*( DFLOAT( INT(NF/2.0D0) )*CQBBB(X3) + CQBBCD(X3) )
      ENDIF
      PDQQBS=HELP / X1 / X2
      RETURN
      END

      DOUBLE PRECISION FUNCTION FINT0(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for VEGAS.
C.
C.    Born cross-section.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
C.
      X=XV(1)
      FINT0=PDQQB(X,TAU/X) / X
      RETURN
      END

      DOUBLE PRECISION FUNCTION FINT1(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for Vegas.
C.
C.    1st order, q qb, singular terms.
C.
C.    Folding of : PDQQB*CQB1
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
C.
      Z=XV(1)
      X=XV(2)
      W=( TAU*(1.0D0-Z) + (Z-TAU)*X ) / (1.0D0-TAU)
      FINT1=( (Z-TAU) / Z / (TAU*(1.0D0-Z)+(Z-TAU)*X) * PDQQB(W/Z,TAU/W)
     .        -PDQQB(X,TAU/X) / X ) * CQB1(Z)
      RETURN
      END

      DOUBLE PRECISION FUNCTION FINT2(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for Vegas.
C.
C.    2nd order, q qb, CF**2, singular terms.
C.
C.    Folding of : PDQQB*CQBCF
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
C.
      Z=XV(1)
      X=XV(2)
      W=( TAU*(1.0D0-Z) + (Z-TAU)*X ) / (1.0D0-TAU)
      FINT2=( (Z-TAU) / Z / (TAU*(1.0D0-Z)+(Z-TAU)*X) * PDQQB(W/Z,TAU/W)
     .        -PDQQB(X,TAU/X) / X ) * CQBCF(Z)
      RETURN
      END

      DOUBLE PRECISION FUNCTION FINT3(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for Vegas.
C.
C.    2nd order, q qb, CA*CF, singular terms.
C.
C.    Folding of : PDQQB*CQBCA
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
C.
      Z=XV(1)
      X=XV(2)
      W=( TAU*(1.0D0-Z) + (Z-TAU)*X ) / (1.0D0-TAU)
      FINT3=( (Z-TAU) / Z / (TAU*(1.0D0-Z)+(Z-TAU)*X) * PDQQB(W/Z,TAU/W)
     .        -PDQQB(X,TAU/X) / X ) * CQBCA(Z)
      RETURN
      END

      DOUBLE PRECISION FUNCTION FINT4(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for Vegas.
C.
C.    2nd order, q qb, NF*CF, singular terms.
C.
C.    Folding of : PDQQB*CQBNF
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
C.
      Z=XV(1)
      X=XV(2)
      W=( TAU*(1.0D0-Z) + (Z-TAU)*X ) / (1.0D0-TAU)
      FINT4=( (Z-TAU) / Z / (TAU*(1.0D0-Z)+(Z-TAU)*X) * PDQQB(W/Z,TAU/W)
     .        -PDQQB(X,TAU/X) / X ) * CQBNF(Z)
      RETURN
      END

      DOUBLE PRECISION FUNCTION FINT5(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for Vegas.
C.
C.    1st order, q qb, regular terms.
C.
C.    Folding of : PDQQB*CQB1R
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
C.
      Z=XV(1)
      X=XV(2)
      W=( TAU*(1.0D0-Z) + (Z-TAU)*X ) / (1.0D0-TAU)
      FINT5=(Z-TAU) / Z / (TAU*(1.0D0-Z)+(Z-TAU)*X) * PDQQB(W/Z,TAU/W) *
     .      CQB1R(Z)
      RETURN

      END
      DOUBLE PRECISION FUNCTION FINT6(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for Vegas.
C.
C.    1st order, q g
C.
C.    Folding of : PDQG*CQG1
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
C.
      Z=XV(1)
      X=XV(2)
      W=( TAU*(1.0D0-Z) + (Z-TAU)*X ) / (1.0D0-TAU)
      FINT6=(Z-TAU) / Z / (TAU*(1.0D0-Z)+(Z-TAU)*X) * PDQG(W/Z,TAU/W) *
     .      CQG1(Z)
      RETURN
      END

      DOUBLE PRECISION FUNCTION FINT7(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for Vegas.
C.
C.    2nd order, q qb, regular terms.
C.
C.    Folding of : PDQQB*( CQBCFR + CQBCAR + CQBNFR + 2*CQBACD )
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
      COMMON/FACREN/FACLM,RENLR,NATFRS
      LOGICAL NATFRS
C.
      Z=XV(1)
      X=XV(2)
      W=( TAU*(1.0D0-Z) + (Z-TAU)*X ) / (1.0D0-TAU)
      HELP=CQBCFR(Z) + CQBCAR(Z) + CQBNFR(Z) + 2.0D0*CQBACD(Z)
      FINT7=(Z-TAU) / Z / (TAU*(1.0D0-Z)+(Z-TAU)*X) * PDQQB(W/Z,TAU/W) *
     .      HELP
      RETURN
      END

      DOUBLE PRECISION FUNCTION FINT8(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for Vegas.
C.
C.    2nd order, g g
C.
C.    Folding of : PDGG*CGG
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
C.
      Z=XV(1)
      X=XV(2)
      W=( TAU*(1.0D0-Z) + (Z-TAU)*X ) / (1.0D0-TAU)
      FINT8=(Z-TAU) / Z / (TAU*(1.0D0-Z)+(Z-TAU)*X) * PDGG(W/Z,TAU/W) *
     .      CGG(Z)
      RETURN
      END

      DOUBLE PRECISION FUNCTION FINT9(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for Vegas.
C.
C.    2nd order, q q + qb qb
C.
C.    For functions to be folded see : subroutine PDQQ
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
C.
      Z=XV(1)
      X=XV(2)
      W=( TAU*(1.0D0-Z) + (Z-TAU)*X ) / (1.0D0-TAU)
      FINT9=(Z-TAU) / Z / (TAU*(1.0D0-Z)+(Z-TAU)*X) * PDQQ(W/Z,TAU/W,Z)
      RETURN
      END

      DOUBLE PRECISION FUNCTION FINT10(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for Vegas.
C.
C.    2nd order, q qb, singlet contribution
C.
C.    for functions to be folded see : subroutine PDQQBS
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
C.
      Z=XV(1)
      X=XV(2)
      W=( TAU*(1.0D0-Z) + (Z-TAU)*X ) / (1.0D0-TAU)
      FINT10=(Z-TAU) / Z / (TAU*(1.0D0-Z)+(Z-TAU)*X) *
     .       PDQQBS(W/Z,TAU/W,Z)
      RETURN
      END

      DOUBLE PRECISION FUNCTION FINT11(XV,WGT)
C.----------------------------------------------------------------------
C.
C.    Input for Vegas.
C.
C.    2nd order, q g
C.
C.    Folding of : PDQG*CQG2
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      DIMENSION XV(10)
      COMMON/ENERGY/S,TAU
C.
      Z=XV(1)
      X=XV(2)
      W=( TAU*(1.0D0-Z) + (Z-TAU)*X ) / (1.0D0-TAU)
      FINT11=(Z-TAU) / Z / (TAU*(1.0D0-Z)+(Z-TAU)*X) * PDQG(W/Z,TAU/W) *
     .       CQG2(Z)
      RETURN
      END

      SUBROUTINE INTVEG()
C.----------------------------------------------------------------------
C.
C.    Integration of the functions FINT* using VEGAS.
C.
C.    The results are put into :
C.
C.    BORN   : Born cross-section
C.    RES(1) : q qb, 1st order, singular terms
C.    RES(2) : q qb, 2nd order, CF**2, singular terms
C.    RES(3) : q qb, 2nd order, CA*CF, singular terms
C.    RES(4) : q qb, 2nd order, nf*CF, singular terms
C.    RES(5) : q qb, 1st order, regular
C.    RES(6) : q g , 1st order
C.    RES(7) : q qb, 2nd order, regular
C.    RES(8) : g g ,
C.    RES(9) : q q + qb qb
C.    RES(10): q qb, singlet part
C.    RES(11): q g , 2nd order
C.
C.    The standard deviations are put into BFOUT, FOUT(i)
C.
C.    The parameters used by VEGAS are initialized by the subroutine
C.    KEUZE, see COMMON block PVEGAS.
C.
C.    The COMMON blocks FLAG and ORDE control which integrals are
C.    evaluated by calling INTVEG. These COMMON blocks are set by
C.    the subroutine KEUZE.
C.
C.    The COMMON block BVEG1 is used to communicate with the VEGAS
C.    program.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/ENERGY/S,TAU
      COMMON/FLAG/REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
      LOGICAL REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
      COMMON/ORDE/NORDER
      COMMON/BVEG1/XL(10),XU(10),ACC,NDIM,NCALL,ITMX,NPRN
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/LOWORD/BORN,BFOUT
      COMMON/PVEGAS/NPRINT,IT1,IT2,N1,N2
      DIMENSION IT1(0:20),IT2(0:20),N1(0:20),N2(0:20)
      EXTERNAL FINT0,FINT1,FINT2,FINT3,FINT4,FINT5,FINT6,FINT7,FINT8,
     .         FINT9,FINT10,FINT11
C.
C.    Initialization of some VEGAS parameters.
C.
      DO 1 K=1,10
        XL(K)=TAU
        XU(K)=1.0D0
    1 CONTINUE
      ACC=-1.0D0
      NPRN=NPRINT
C.
C.    Born is a one dimensional integral
C.
      NDIM=1
      ITMX=IT1(0)
      NCALL=N1(0)
      CALL VEGAS(FINT0,R,F,X)
      ITMX=IT2(0)
      NCALL=N2(0)
      CALL VEGAS1(FINT0,R,F,X)
      BORN=R
      BFOUT=F/R
C.
C.    Other integrals are two dimensional
C.
      NDIM=2
      IF (NORDER.GT.0) THEN
        ITMX=IT1(1)
        NCALL=N1(1)
        CALL VEGAS(FINT1,R,F,X)
        ITMX=IT2(1)
        NCALL=N2(1)
        CALL VEGAS1(FINT1,R,F,X)
        RES(1)=R
        FOUT(1)=F/R
        IF (NORDER.GT.1) THEN
          ITMX=IT1(2)
          NCALL=N1(2)
          CALL VEGAS(FINT2,R,F,X)
          ITMX=IT2(2)
          NCALL=N2(2)
          CALL VEGAS1(FINT2,R,F,X)
          RES(2)=R
          FOUT(2)=F/R
          ITMX=IT1(3)
          NCALL=N1(3)
          CALL VEGAS(FINT3,R,F,X)
          ITMX=IT2(3)
          NCALL=N2(3)
          CALL VEGAS1(FINT3,R,F,X)
          RES(3)=R
          FOUT(3)=F/R
          ITMX=IT1(4)
          NCALL=N1(4)
          CALL VEGAS(FINT4,R,F,X)
          ITMX=IT2(4)
          NCALL=N2(4)
          CALL VEGAS1(FINT4,R,F,X)
          RES(4)=R
          FOUT(4)=F/R
        ENDIF
      ENDIF
      IF (REG1) THEN
      IF(NORDER.GT.0) THEN
        ITMX=IT1(5)
        NCALL=N1(5)
        CALL VEGAS(FINT5,R,F,X)
        ITMX=IT2(5)
        NCALL=N2(5)
        CALL VEGAS1(FINT5,R,F,X)
        RES(5)=R
        FOUT(5)=F/R
        ELSE
        RES(5)=0.0D0
        ENDIF
      ENDIF
      IF (QUGL1) THEN
      IF(NORDER.GT.0) THEN
        ITMX=IT1(6)
        NCALL=N1(6)
        CALL VEGAS(FINT6,R,F,X)
        ITMX=IT2(6)
        NCALL=N2(6)
        CALL VEGAS1(FINT6,R,F,X)
        RES(6)=R
        FOUT(6)=F/R
       ELSE
       RES(6)=0.0D0
       ENDIF
      ENDIF
      IF (REG2) THEN
      IF(NORDER.GT.1) THEN        
        ITMX=IT1(7)
        NCALL=N1(7)
        CALL VEGAS(FINT7,R,F,X)
        ITMX=IT2(7)
        NCALL=N2(7)
        CALL VEGAS1(FINT7,R,F,X)
        RES(7)=R
        FOUT(7)=F/R
       ELSE
       RES(7)=0.0D0
      ENDIF
      ENDIF
      IF (GLGL) THEN
      IF(NORDER.GT.1) THEN        
        ITMX=IT1(8)
        NCALL=N1(8)
        CALL VEGAS(FINT8,R,F,X)
        ITMX=IT2(8)
        NCALL=N2(8)
        CALL VEGAS1(FINT8,R,F,X)
        RES(8)=R
        FOUT(8)=F/R
        ELSE
        RES(8)=0.0D0
      ENDIF
      ENDIF
      IF (QUQU) THEN
      IF(NORDER.GT.1) THEN                
        ITMX=IT1(9)
        NCALL=N1(9)
        CALL VEGAS(FINT9,R,F,X)
        ITMX=IT2(9)
        NCALL=N2(9)
        CALL VEGAS1(FINT9,R,F,X)
        RES(9)=R
        FOUT(9)=F/R
      ELSE
      RES(9)=0.0D0
      ENDIF
      ENDIF
      IF (QUQUBS) THEN
      IF(NORDER.GT.1) THEN                
        ITMX=IT1(10)
        NCALL=N1(10)
        CALL VEGAS(FINT10,R,F,X)
        ITMX=IT2(10)
        NCALL=N2(10)
        CALL VEGAS1(FINT10,R,F,X)
        RES(10)=R
        FOUT(10)=F/R
      ELSE
      RES(10)=0.0D0
      ENDIF
      ENDIF
      IF (QUGL2) THEN
      IF(NORDER.GT.1) THEN                
        ITMX=IT1(11)
        NCALL=N1(11)
        CALL VEGAS(FINT11,R,F,X)
        ITMX=IT2(11)
        NCALL=N2(11)
        CALL VEGAS1(FINT11,R,F,X)
        RES(11)=R
        FOUT(11)=F/R
      ELSE
      RES(11)=0.0D0
      ENDIF
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION ZEROTH()
C.----------------------------------------------------------------------
C.
C.    The Born cross-section
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/LOWORD/BORN,BFOUT
C.
      ZEROTH=BORN * FACTOR
      RETURN
      END

      DOUBLE PRECISION FUNCTION CONS1()
C.----------------------------------------------------------------------
C.
C.    The delta function part of the 1st order, q qb, singular Drell-Yan
C.    correction term.
C.
C.    To include the factorization scale dependent contributions, set
C.    the flag NATFRS equal to .FALSE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/LOWORD/BORN,BFOUT
      COMMON/FACREN/FACLM,RENLR,NATFRS
      COMMON/ORDE/NORDER
      LOGICAL NATFRS
C.
      IF(NORDER.LT.1) THEN
      CONS1=0.0D0
      ELSE
      HELP=8.0D0*ZETA2 - 16.0D0
      IF (.NOT.NATFRS) HELP=HELP + 6.0D0*FACLM
      CONS1=BORN * FACTOR * ALSQ * CF * HELP
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION PART1()
C.----------------------------------------------------------------------
C.
C.    The log(1-tau)**i type contribution from the 1st order, q qb,
C.    singular Drell-Yan correction term.
C.
C.    To include the factorization scale dependent contributions, set
C.    the flag NATFRS equal to .FALSE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/LOWORD/BORN,BFOUT
      COMMON/ENERGY/S,TAU
      REAL*8 LTMIN
      COMMON/FACREN/FACLM,RENLR,NATFRS
      COMMON/ORDE/NORDER
      LOGICAL NATFRS
C.
      IF(NORDER.LT.1) THEN
      PART1=0.0D0
      ELSE
      LTMIN=DLOG(1.0D0-TAU)
      HELP=8.0D0*LTMIN**2
      IF (.NOT.NATFRS) HELP=HELP + 8.0D0*LTMIN*FACLM
      PART1=BORN * FACTOR * ALSQ * CF * HELP
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION REST1()
C.----------------------------------------------------------------------
C.
C.    The contribution from the VEGAS integration over the Di(x) terms
C.    in the 1st order, q qb. singular Drell-Yan correction term.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/ORDE/NORDER
C.
      IF(NORDER.LT.1) THEN
      REST1=0.0D0
      ELSE
      REST1=FACTOR * ALSQ * RES(1)
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION HARD1()
C.----------------------------------------------------------------------
C.
C.    The 1st order, q qb, regular contribution.
C.
C.    To use this function the flag REG1 has to be equal to .TRUE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/ORDE/NORDER
      COMMON/FLAG/REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
      LOGICAL REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
C.
      IF(NORDER.LT.1) THEN
      HARD1=0.0D0
      ELSE
      IF (.NOT.REG1) THEN
        HARD1=0.0D0
      ELSE
        HARD1=FACTOR * ALSQ * RES(5)
      ENDIF
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION QUAGLU()
C.----------------------------------------------------------------------
C.
C.    The 1st order q g contribution.
C.
C.    To use this function the flag QUGL1 has to be equal to .TRUE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/ORDE/NORDER
      COMMON/FLAG/REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
      LOGICAL REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
C.
      IF(NORDER.LT.1) THEN
      QUAGLU=0.0D0
      ELSE
      IF (.NOT.QUGL1) THEN
        QUAGLU=0.0D0
      ELSE
        QUAGLU=FACTOR * ALSQ * RES(6)
      ENDIF
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION CONSCF()
C.----------------------------------------------------------------------
C.
C.    The delta function part of the 2nd order, q qb, singular Drell-Yan
C.    correction term. Only the CF**2 colour part is taken into account.
C.
C.    To include the factorization scale dependent contributions, set
C.    the flag NATFRS equal to .FALSE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/LOWORD/BORN,BFOUT
      COMMON/FACREN/FACLM,RENLR,NATFRS
      COMMON/ORDE/NORDER
      LOGICAL NATFRS
C.
      IF(NORDER.LT.2) THEN
      CONSCF=0.0D0
      ELSE
      HELP=8.0D0/5.0D0*ZETA2**2 - 60.0D0*ZETA3 - 70.0D0*ZETA2 +
     .     511.0D0/4.0D0
      IF (.NOT.NATFRS) THEN
        HELP=HELP + ( 18.0D0 - 32.0D0*ZETA2 )*FACLM**2 +
     .       ( -93.0D0 + 24.0D0*ZETA2 + 176.0D0*ZETA3 )*FACLM
      ENDIF
      CONSCF=BORN * FACTOR * ALSQ**2 * CF**2 * HELP
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION PARTCF()
C.----------------------------------------------------------------------
C.
C.    The log(1-tau)**i type contribution from the 2nd order, q qb,
C.    singular Drell-Yan correction term. Only the CF**2 colour part is
C.    taken into account.
C.
C.    To include the factorization scale dependent contributions, set
C.    the flag NATFRS equal to .FALSE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/LOWORD/BORN,BFOUT
      COMMON/ENERGY/S,TAU
      REAL*8 LTMIN
      COMMON/FACREN/FACLM,RENLR,NATFRS
      COMMON/ORDE/NORDER
      LOGICAL NATFRS
C.
      IF(NORDER.LT.2) THEN      
      PARTCF=0.0D0
      ELSE      
      LTMIN=DLOG(1.0D0-TAU)
      HELP=32.0D0*LTMIN**4 - ( 64.0D0*ZETA2 + 128.0D0 )*LTMIN**2 +
     .     256.0D0*ZETA3*LTMIN
      IF (.NOT.NATFRS) THEN
        HELP=HELP + 64.0D0*FACLM*LTMIN**3 +
     .       ( 32.0D0*FACLM + 48.0D0 )*FACLM*LTMIN**2 +
     .       ( 48.0D0*FACLM - 128.0D0 - 64.0D0*ZETA2 )*FACLM*LTMIN
      ENDIF
      PARTCF=BORN * FACTOR * ALSQ**2 * CF**2 * HELP
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION RESTCF()
C.----------------------------------------------------------------------
C.
C.    The contribution from the VEGAS integration over the Di(x) terms
C.    in the 2nd order, q qb. singular Drell-Yan correction term.
C.    Only the CF**2 colour part is taken into account.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/ORDE/NORDER
C.
      IF(NORDER.LT.2) THEN
      RESTCF=0.0D0
      ELSE
      RESTCF=FACTOR * ALSQ**2 * RES(2)
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION CONSCA()
C.----------------------------------------------------------------------
C.
C.    The delta function part of the 2nd order, q qb, singular Drell-Yan
C.    correction term. Only the CA*CF colour part is taken into account.
C.
C.    To include the factorization and renormalization scale dependent
C.    terms, set the flag NATFRS equal to .FALSE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/LOWORD/BORN,BFOUT
      COMMON/FACREN/FACLM,RENLR,NATFRS
      COMMON/ORDE/NORDER
      LOGICAL NATFRS
C.
      IF(NORDER.LT.2) THEN
      CONSCA=0.0D0
      ELSE
      HELP=-1535.0D0/12.0D0 + 592.0D0/9.0D0*ZETA2 + 28.0D0*ZETA3
     .     -12.0D0/5.0D0*ZETA2**2
      IF (.NOT.NATFRS) THEN
        HELP=HELP + 11.0D0*FACLM**2 - 22.0D0*FACLM*RENLR +
     .       ( 17.0D0/3.0D0 + 88.0D0/3.0D0*ZETA2 - 24.0D0*ZETA3)*FACLM +
     .       ( 176.0D0/3.0D0 - 88.0D0/3.0D0*ZETA2 )*RENLR
      ENDIF
      CONSCA=BORN * FACTOR * ALSQ**2 * CA*CF * HELP
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION PARTCA()
C.----------------------------------------------------------------------
C.
C.    The log(1-tau)**i type contribution from the 2nd order, q qb,
C.    singular Drell-Yan correction term. Only the CA*CF colour part is
C.    taken into account.
C.
C.    To include the factorization and renormalization scale dependent
C.    terms, set the flag NATFRS equal to .FALSE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/LOWORD/BORN,BFOUT
      COMMON/ENERGY/S,TAU
      REAL*8 LTMIN
      COMMON/FACREN/FACLM,RENLR,NATFRS
      COMMON/ORDE/NORDER
      LOGICAL NATFRS
C.
      IF(NORDER.LT.2) THEN
      PARTCA=0.0D0
      ELSE
      LTMIN=DLOG(1.0D0-TAU)
      HELP=-176.0D0/9.0D0*LTMIN**3 +
     .     ( 536.0D0/9.0D0 - 16.0D0*ZETA2 )*LTMIN**2 +
     .     ( 56.0D0*ZETA3 + 176.0D0/3.0D0*ZETA2
     .       -1616.0D0/27.0D0 )*LTMIN
      IF (.NOT.NATFRS) THEN
        HELP=HELP - 88.0D0/3.0D0*RENLR*LTMIN**2 +
     .       ( 44.0D0/3.0D0*FACLM**2 - 88.0D0/3.0D0*FACLM*RENLR +
     .         ( 536.0D0/9.0D0 - 16.0D0*ZETA2 )*FACLM )*LTMIN
      ENDIF
      PARTCA=BORN * FACTOR * ALSQ**2 * CA*CF * HELP
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION RESTCA()
C.----------------------------------------------------------------------
C.
C.    The contribution from the VEGAS integration over the Di(x) terms
C.    in the 2nd order, q qb. singular Drell-Yan correction term.
C.    Only the CA*CF colour part is taken into account.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/ORDE/NORDER
C.
      IF(NORDER.LT.2) THEN
      RESTCA=0.0D0
      ELSE
      RESTCA=FACTOR * ALSQ**2 * RES(3)
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION CONSNF()
C.----------------------------------------------------------------------
C.
C.    The delta function part of the 2nd order, q qb, singular Drell-Yan
C.    correction term. Only the nf*CF colour part is taken into account.
C.
C.    To include the factorization scale dependent contributions, set
C.    the flag NATFRS equal to .FALSE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/LOWORD/BORN,BFOUT
      COMMON/FACREN/FACLM,RENLR,NATFRS
      COMMON/ORDE/NORDER
      LOGICAL NATFRS
C.
      IF(NORDER.LT.2) THEN
      CONSNF=0.0D0
      ELSE
      HELP=8.0D0*ZETA3 - 112.0D0/9.0D0*ZETA2 + 127.0D0/6.0D0
      IF (.NOT.NATFRS) THEN
        HELP=HELP - 2.0D0*FACLM**2 + 4.0D0*FACLM*RENLR
     .       -( 2.0D0/3.0D0 + 16.0D0/3.0D0*ZETA2 )*FACLM +
     .       ( -32.0D0/3.0D0 + 16.0D0/3.0D0*ZETA2 )*RENLR
      ENDIF
      CONSNF=BORN * FACTOR * ALSQ**2 * NF*CF * HELP
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION PARTNF()
C.----------------------------------------------------------------------
C.
C.    The log(1-tau)**i type contribution from the 2nd order, q qb,
C.    singular Drell-Yan correction term. Only the nf*CF colour part is
C.    taken into account.
C.
C.    To include the factorization scale dependent contributions, set
C.    the flag NATFRS equal to .FALSE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/CASI/CF,CA,NF
      REAL*8 NF
      COMMON/ZETA/PI,ZETA2,ZETA3
      COMMON/LOWORD/BORN,BFOUT
      COMMON/ENERGY/S,TAU
      REAL*8 LTMIN
      COMMON/FACREN/FACLM,RENLR,NATFRS
      COMMON/ORDE/NORDER
      LOGICAL NATFRS
C.
      IF(NORDER.LT.2) THEN
      PARTNF=0.0D0
      ELSE
      LTMIN=DLOG(1.0D0-TAU)
      HELP=32.0D0/9.0D0*LTMIN**3 - 80.0D0/9.0D0*LTMIN**2 +
     .     ( 224.0D0/27.0D0 - 32.0D0/3.0D0*ZETA2 )*LTMIN
      IF (.NOT.NATFRS) THEN
        HELP=HELP + 16.0D0/3.0D0*RENLR*LTMIN**2 +
     .       ( -8.0D0/3.0D0*FACLM**2 + 16.0D0/3.0D0*FACLM*RENLR
     .         -80.0D0/9.0D0*FACLM )*LTMIN
      ENDIF
      PARTNF=BORN * FACTOR * ALSQ**2 * NF*CF * HELP
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION RESTNF()
C.----------------------------------------------------------------------
C.
C.    The contribution from the VEGAS integration over the Di(x) terms
C.    in the 2nd order, q qb. singular Drell-Yan correction term.
C.    Only the nf*CF colour part is taken into account.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/ORDE/NORDER
C.
      IF(NORDER.LT.2) THEN
      RESTNF=0.0D0
      ELSE
      RESTNF=FACTOR * ALSQ**2 * RES(4)
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION HARD2()
C.----------------------------------------------------------------------
C.
C.    Contribution from the 2nd order, q qb, regular terms.
C.
C.    To use this function the flag REG2 has to be set equal to .TRUE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/ORDE/NORDER
      COMMON/FLAG/REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
      LOGICAL REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
C.
      IF(NORDER.LT.2) THEN
      HARD2=0.0D0
      ELSE
      IF (.NOT.REG2) THEN
        HARD2=0.0D0
      ELSE
        HARD2=FACTOR * ALSQ**2 * RES(7)
      ENDIF
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION GLUGLU()
C.----------------------------------------------------------------------
C.
C.    The contribution from the g g subprocess.
C.
C.    To use this function the flag GLGL has to be set equal to .TRUE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/ORDE/NORDER
      COMMON/FLAG/REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
      LOGICAL REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
C.
      IF(NORDER.LT.2) THEN
      GLUGLU=0.0D0
      ELSE
      IF (.NOT.GLGL) THEN
        GLUGLU=0.0D0
      ELSE
        GLUGLU=FACTOR * ALSQ**2 * RES(8)
      ENDIF
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION QUAQUA()
C.----------------------------------------------------------------------
C.
C.    Contribution from the q q and qb qb subprocesses.
C.
C.    To use this function the flag QUQU has to be set equal to .TRUE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/ORDE/NORDER
      COMMON/FLAG/REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
      LOGICAL REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
C.
      IF(NORDER.LT.2) THEN
      QUAQUA=0.0D0
      ELSE
      IF (.NOT.QUQU) THEN
        QUAQUA=0.0D0
      ELSE
        QUAQUA=FACTOR * ALSQ**2 * RES(9)
      ENDIF
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION QUQUBA()
C.----------------------------------------------------------------------
C.
C.    Contribution from the q qb, singlet subprocess.
C.
C.    To use this function the flag QUQUBS has to be set equal to .TRUE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/ORDE/NORDER
      COMMON/FLAG/REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
      LOGICAL REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
C.
      IF(NORDER.LT.2) THEN
      QUQUBA=0.0D0
      ELSE
      IF (.NOT.QUQUBS) THEN
        QUQUBA=0.0D0
      ELSE
        QUQUBA=FACTOR * ALSQ**2 * RES(10)
      ENDIF
      ENDIF
      RETURN
      END

      DOUBLE PRECISION FUNCTION QUAGL2()
C.----------------------------------------------------------------------
C.
C.    The 2nd order q g contribution.
C.
C.    To use this function the flag QUGL2 has to be equal to .TRUE.
C.
C.----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C.
      COMMON/KOP/ALSQ,ALSQT,FACTOR
      COMMON/RESULT/RES(20),FOUT(20)
      COMMON/ORDE/NORDER
      COMMON/FLAG/REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
      LOGICAL REG1,QUGL1,EXPECT,REG2,GLGL,QUQU,QUQUBS,QUGL2
C.
      IF(NORDER.LT.2) THEN
      QUAGL2=0.0D0
      ELSE
      IF (.NOT.QUGL2) THEN
        QUAGL2=0.0D0
      ELSE
        QUAGL2=FACTOR * ALSQ**2 * RES(11)
      ENDIF
      ENDIF
      RETURN
      END

